#!/usr/bin/perl -w
# $Id: ACSd.pl,v 1.4 2002/04/16 16:09:00 santana Exp $
#
# ACSd - Application Cache Server
#
# An implementation of the ACS component of macs
# 
# 2.99.BAM
# 7.99.mario
#

package ACSd;
use strict;
use lib "../lib";
use Tie::DistHash;
use Net::muxDaemon;

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die('Couldn\'t configure')
    unless $cfg->Sections(); 
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}

# make sure we're running in the background
FORK: {
    # the process id variable
    my($pid);
    # spawn a child
    if($pid = fork()) {
	# parent process here, bye bye
	exit(0);
    } elsif(defined $pid) {
	# child process here, we continue
	last FORK;
    } elsif($! =~ m/[Nn]o more process/) {
	# can't fork, sleep and try again
	sleep(5);
	redo FORK;
    } else {
	# unrecoverable fork error
	die("Couldn't fork: $!\n");
    }
}

#
# The hosts in our distributed setup
my($hosts) = [];
for ( $cfg->val('ACS','p') ) {
    push(@$hosts, {addr => $_});
}

#
# the DistHash database
my($db) = {};
my($ch) = tie(%$db, 'Tie::DistHash',
	      ckptfile => $cfg{ACS}{ckptf},
	      sync => $hosts);

#
# vroom...
new Net::muxDaemon( POOL => $hosts,
		    database => $db,
		    cache => $ch,
		    LOGTAG => $cfg{ACS}{logtag},
		    PIDFILE => $cfg{ACS}{pidf}
		 );
die("Net::muxDaemons don't return!\n\t$!\n\t$@\n");

#
#
#
sub process_command {
    my($this,$buf) = (shift(),shift());
    if ( $buf =~ m/^\s*$/ ) {
	syslog('debug','Exiting process_command: OK NOOP');
	return "OK NOOP\n";
    }  
    # used all over
    my($reply,$skey,$aid,$bc,$adm,$operation);
    # the operation to perform
    ($operation,$buf) = split(' ', $buf, 2);
    $operation = uc($operation);
    $buf =~ s/^\s+|\s+$//g; # eat {lead,trail}ing whitespace

    syslog('debug',"Entering process_command: $operation $buf");

  OP: {

      # PUT aid skey bc -- adm is bc bytes more after all this
      if ( $operation eq 'PUT' ) {
	  ($aid,$skey,$bc) = split(' ',$buf,3);
	  unless ( $aid && $skey && defined($bc) ) {
	      $reply = "ERR PUT: not enough arguments\n";
	      last OP;
	  }
	  my($client) = shift();
	  # get the Application Data Morsel
	  unless ( $this->getmore($client,$bc,\$adm) ) {
	      $reply = "ERR $?\n";
	      last OP;
	  }
	  if ( exists($this->{database}{$skey}) ) {	
	      $this->{database}{$skey}{$aid} = [$bc, $adm];
	      $this->{cache}->sync($skey);
	  } else {	
	      $this->{database}{$skey} = { $aid => [$bc, $adm] };
	  }
	  syslog('debug',"Put ADM .$adm.");
	  $reply = "OK\n";
      }

      # DEL skey -- deletes skey
      elsif ( $operation eq 'DEL' ) {
	  ($skey,$buf) = split(' ',$buf,2);
	  unless ( $skey ) {
 	      $reply = "ERR DEL: not enough arguments\n";
	      last OP;
	  }
	  if ( defined($buf) && $buf =~ /\S/ ) {
	      $reply = 'ERR DEL: too many arguments';
	      last OP;
	  }
	  delete($this->{database}{$skey});
	  $reply = "OK\n";
      }

      # GET aid skey
      elsif ( $operation eq "GET" ) {
	  ($aid,$skey,$buf) = split(" ",$buf,3);
	  unless ( $aid && $skey ) {
	      $reply = "ERR GET: not enough arguments\n";
	      last OP;
	  }
	  if ( defined($buf) && $buf =~ /\S/ ) {
	      $reply = "ERR GET: too many arguments\n";
	      last OP;
	  }
	  
	  if ( exists($this->{database}{$skey}) &&
	       defined($bc = $this->{database}{$skey}{$aid}) ) {
	      $reply = "YES $bc->[0]\n$bc->[1]";
	  } else {
	      $reply = "NO\n";
	  }
      }

      # PING
      elsif ( $operation eq 'PING' ) {
	  $reply = "PONG\n";
      }

      # QUIT
      elsif ( $operation eq 'QUIT' ) {
	  $KILLED = 1;
	  $reply = "OK\n";
      }

      # menospeakie
      else {
	  $reply = "ERR unknown command: $operation $buf\n"
	  }

  } # OP

    syslog('debug',"Exiting process_command: $reply");
    return $reply;
} # sub process_command

