#!/usr/bin/perl -w
#
# test_ACSd - Application Cache Server test client
#
# 2.99.BAM
# 7.99.mario
#

package test_ACSd;
use IO::Socket;
use Benchmark;

use Config::IniFiles;

my($cfg,%cfg);
die('Couldn\'t configure')
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => '../macs.cfg');

my($POOL) = [];
for ( $cfg->val('ACS','p') ) {
    push(@$POOL, {addr => $_});
}

mkconns();
populate();

if ( defined($a = $ARGV[0]) ) {
  if ( $a eq "q" ) {
    quit();
    exit 0;
  } elsif ( $a eq "p" ) {
    populate();
    exit 0;
  } elsif ( $a eq "." ) {
    ping();
    exit 0;
  } elsif ( $a eq "c" ) {
    clear();
    exit 0;
  }
}

foreach $s ( @$POOL ) {
  print "\n\nFor ",$s->{addr},"\n";
  timethese(1000, { ping => 'ping($s)',
		    get  => 'get($s)' } 
	    );
}

exit 0;


sub mkconns {
  for ( $c=0; $c<=$#$POOL; $c++ ) {
    $POOL->[$c]{sock} = new IO::Socket::INET(PeerAddr => $POOL->[$c]{addr},
					     Proto    => 'tcp');
    if ( !$POOL->[$c]{sock} ) {
      die "Error connecting: $!";
    } else {
      $POOL->[$c]{sock}->autoflush(1);
    }
  }
}

sub quit {
  foreach $_sock ( @$POOL ) {
    next
      unless $sock = $_sock->{sock};
    $addr = $_sock->{addr};
    $writ = "QUIT\n";
    print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
    print "Resp: ";
    print $sock->getline(),"\n\n";
  }
}

sub populate {
  foreach $_sock ( @$POOL ) {
    next
      unless $sock = $_sock->{sock};
    $addr = $_sock->{addr};
    $writ = "PUT marioaid marioskey".$addr." 12\n";
    print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
    $writ = "thisistheadm";
    print "Sent: (",$sock->write($writ,length $writ),") ",$writ, "\n";
    print "Resp: ", $sock->getline(),"\n\n";
  }
}

sub clear {
  foreach $_sock ( @$POOL ) {
    next
      unless $sock = $_sock->{sock};
    $addr = $_sock->{addr};
    $writ = "DEL marioskey".$_sock->{addr}."\n";
    print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
    print "Resp: ", $sock->getline(),"\n\n";
  }
}

sub ping {
  $server = shift();
  $writ = "PING\n";
  unless ( $server->{sock}->write($writ,length $writ) != length $writ ) {
    print "PING failed.\n";
  }
  unless ( $server->{sock}->getline() eq "PONG\n" ) {
    print "Bad PONG\n";
  }
}

sub get {
  $server = shift();
  $writ = "GET marioaid marioskey".$server->{addr}."\n";
  $server->{sock}->write($writ,length $writ);
  $read = $server->{sock}->getline();
  $read =~ /((\S+)(?:\s+([0-9]+))?\n)/;
  $read = '';
  if ( $2 eq "YES" && $3 > 0 ) {
    $server->{sock}->read($read,$3);
  } else {
    print "$1 $2\n";
  }
}



######
# Old tests -- this code here for... uh... historical purposes?
######

die;

foreach $_sock ( @$POOL ) {
  next
    unless $sock = $_sock->{sock};
  $addr = $_sock->{addr};
  
  if ( defined($ARGV[0]) && $ARGV[0] eq 'q' ) { 
    $writ = "QUIT\n";
    print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
    print "Resp: ";
    print $sock->getline(),"\n\n";
    next;
  }
  
  #$writ = "PING\n";
  #print "Sent: ";
  #unless ( $sock->write($writ,length $writ) != length $writ ) {
  #  print "(failed!) ";
  #}
  #print $writ;
  #print "Resp: ";
  #print $sock->getline(),"\n\n";

  #$writ = "PUT marioaid marioskey".$host.$port." 12\n";
  #print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
  #$writ = "thisistheadm";
  #print "Sent: (",$sock->write($writ,length $writ),") ",$writ, "\n";
  #print "Resp: ", $sock->getline(),"\n\n";
  
  #$writ = "DEL marioskey".$_sock->{host}.$_sock->{port}."\n";
  #print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
  #print "Resp: ", $sock->getline(),"\n\n";

  $iter = 0;
  for $__sock( @$POOL ) {
    $host = $__sock->{host};
    $port = $__sock->{port};
    for($count=0; $count < 1000; $count++) {
      $writ = "GET marioaid marioskey".$host.$port."\n";
      #print "Sent: (",$sock->write($writ,length $writ),") ",$writ;
      $sock->write($writ,length $writ);
      #print "Resp: ";
      $read = $sock->getline();
      #print $read;
      $read =~ /((\S+)(?:\s+([0-9]+))?\n)/;
      $read = '';
      if ( $2 eq "YES" && $3 > 0 ) {
	#print "ADM (",$sock->read($read,$3),") ",$read,"\n";
	print ".";
	$sock->read($read,$3);
      } else {
	print ":";
      }
      $iter++;
    }
  }
  
  @t2 = times;
  $dur = ($t2[0]+$t2[1]) - ($t1[0]+$t1[1]);
  $per = $dur / $iter;
  print "for ",$_sock->{host},":",$_sock->{port},
        " (",$dur," secs for ",$iter," gets -- ",$per," secs per get)\n";
  print "user\tsys\tcuser\tcsys\n",join("\t",@t1),"\n",join("\t",@t2),"\n\n\n";
}


