#if !defined(AFX_ISAPIAUC_H__F5045690_9F3D_11D3_923D_00008635FB59__INCLUDED_)
#define AFX_ISAPIAUC_H__F5045690_9F3D_11D3_923D_00008635FB59__INCLUDED_

#include "resource.h"
#include "iostream.h" // consts for...
#include "fstream.h"  // ...file io classes
#include "Afxsock.h"  // MFC sock includes
#include "Afxmt.h"    // MFC mt includes

#define AUC_BUFSIZ 4096
#define AUC_COOKIE_NAME "test-adsk-waca-skey"
#define AUC_WACA_HEADER "test-adsk-waca-uid"
#define AUC_LIA_URL "http://ernie.autodesk.com:8080/cgi-bin/LIA.pl"
#define AUS_DEFAULTPORT 4322

// linked list of AUS connections
typedef struct ll_ausconns_st {
	char *host;
	int port;
	int isup;
	int inuse;
	CSocket *sock;
	CSocketFile *file;
	CArchive *in;
	CArchive *out;
	struct ll_ausconns_st* next;
} ll_ausconns;

// linked list of secure urls
typedef struct ll_securls_st {
	char *url;
	int isxp;
	size_t len;
	struct ll_securls_st* next;
} ll_securls;

// socket_watcher thread controlling function
// FIXME: if we were to create our own thread
//   rather than let AfxBeginThread to do it
//   for us, we'd have a much better OO model
//   and prolly more speed.
UINT __cdecl auc_socket_watcher(LPVOID);

class CISAPIAUCFilter : public CHttpFilter {

private:

	// the linked lists of secure urls
	ll_securls *securls;

	// the socket_watcher thread
	CWinThread *auc_socket_watcher_thread;
	AFX_THREADPROC fp_auc_socket_watcher;

	// internal utility methods
	int add_securl(char *);
	int add_ausconn(char *);
	DWORD auc_to_LIA(CHttpFilterContext *, PHTTP_FILTER_PREPROC_HEADERS);

public:
	CISAPIAUCFilter();
	~CISAPIAUCFilter();

	// the following are public for use by the socket_watcher
	int auc_socket_watcher_kill;
	// locks for thread-safe use of linked lists
	CCriticalSection *ausconns_lock;
	CCriticalSection *securls_lock;
	// the linked lists of AUS connections
	ll_ausconns *ausconns;
	// utility methods
	int aus_connect(ll_ausconns *);
	int aus_ping(ll_ausconns *);

// Overrides
	// ClassWizard generated virtual function overrides
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//{{AFX_VIRTUAL(CISAPIAUCFilter)
	public:
	virtual BOOL GetFilterVersion(PHTTP_FILTER_VERSION pVer);
	virtual DWORD OnPreprocHeaders(CHttpFilterContext* pCtxt, PHTTP_FILTER_PREPROC_HEADERS pHeaderInfo);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CISAPIAUCFilter)
	//}}AFX_MSG

}; // Class CISAPIAUCFilter

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_ISAPIAUC_H__F5045690_9F3D_11D3_923D_00008635FB59__INCLUDED)
