#!/usr/bin/perl -w
# $Id: AUSd.pl,v 1.7 2002/01/30 02:22:03 santana Exp $
#
# AUSd - Authentication Cache Server
#
# An implementation of the AUS component of macs
# 
# 2.99.BAM
# 7.99.mario
#
# don't pollute the global name space
package AUSd;
use strict;
use lib "../lib";
use Carp;
use Tie::DistHash;
use Net::muxDaemon;

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die('Couldn\'t configure')
    unless $cfg->Sections(); 
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}

# make sure we're running in the background
$SIG{CHLD} = 'IGNORE'; # let all children die
FORK: {
    # the process id variable
    my($pid);
    # spawn a child
    if ($pid = fork()) {
	# parent process here, bye bye
	exit(0);
    } elsif (defined $pid) {
	# child process here, we continue
	last FORK;
    } elsif ($! =~ m/[Nn]o more process/) {
	# can't fork, sleep and try again
	sleep(5);
	redo FORK;
    } else {
	# unrecoverable fork error
	die('Couldn\'t fork');
    }
}

#
# write a config file for the auc
open(CCF, '>../auc.cfg');
print CCF 'LIA "',$cfg{cgi}{LIA},"\"\n";
print CCF 'COOKIE_NAME ',$cfg{AUS}{cookie_name},"\n";
print CCF 'COOKIE_DOMAIN ',$cfg{AUS}{cookie_domain},"\n";
print CCF 'HTTP_UID_HEADER ',$cfg{AUS}{http_uid_header},"\n";
print CCF 'TIMEOUT ',$cfg{AUS}{timeout},"\n";
#
# The hosts in our distributed setup
my($hosts) = [];
for ( $cfg->val('AUS','p') ) {
    print CCF 'AUS ',$_,"\n";
    push(@$hosts, {addr => $_});
}
close CCF;

#
# the DistHash database
my($db) = {};
my($dh) = tie( %$db, 'Tie::DistHash',
	       ckptfile => $cfg{AUS}{ckptf},
	       sync => $hosts
	       );

#
# vroom...
new Net::muxDaemon( POOL => $hosts,
		    database => $db,
		    cache => $dh,
		    LOGTAG => $cfg{AUS}{logtag},
		    PIDFILE => $cfg{AUS}{pidf}
		    );
die("Net::muxDaemons don't return!\n\t$!\n\t$@\n");

my($ats) = undef;

# log folks into the ats
sub do_ats_login {
    my($uid) = shift();
    syslog('debug',"about to log $uid into ats");
    unless ( defined($ats) && $ats->sockport()
	 && $ats->print("LOGIN $uid\n") ) {
	$ats = new IO::Socket::INET( PeerAddr => $cfg{ATS}{s} );
	unless ( defined($ats) && $ats->sockport() &&
	     $ats->print("LOGIN $uid\n") ) {
	    syslog('err',"Couldn't connect to ATS at $cfg{ATS}{s}");
	    return;
	}
    }
    my($r) = $ats->getline(); chomp($r);
    if ( $r =~ /^OK /i ) {
	syslog('debug',"$uid logged into ATS: $r");
    } else {
	syslog('err',"Couldn't log $uid into ATS: $r");
    }
} # sub do_ats_login

# log folks out of the ats
sub do_ats_logout {
    my($uid) = shift();
    if ( !defined($ats) ||  !$ats->sockport()
	 || !$ats->print("LOGOUT $uid\n") ) {
	$ats = new IO::Socket::INET( PeerAddr => $cfg{ATS}{s} );
	if ( !defined($ats) || !$ats->sockport() 
	     || !$ats->print("LOGOUT $uid\n") ) {
	    syslog('err',"Couldn't connect to ATS at $cfg{ATS}{s}");
	    return;
	}
    }
    my($r) = $ats->getline(); chomp($r);
    if ( $r ne 'OK' ) {
	syslog('err',"Couldn't log $uid out of ATS: $r");
    }
} # sub do_ats_logout


#
# FIXME: this still needs to delete ACS entries when a DEL happens.
#
sub process_command {
    my($this,$buf) = (shift(),shift());
    syslog('debug',"Entering process_command: $buf");
    my($Aryref);
    my(@Ary,@command);
    my($skey,$uid,$to,$cip,$reply,$operation);
    if ( $buf =~ m/^\s*$/ ) {
	syslog('debug','Exiting process_command: OK NOOP');
	return "OK NOOP\n";
    }  
    
    # trim the white space off the command string
    $buf =~ s/^\s*|\s*$//;

    # split up the command string on spaces
    @command = split(' ', $buf);
    # the operation to perform
    $operation = uc(shift(@command));


  OP: {

      # PUT skey uid timeout clientip
      if ( $operation eq 'PUT' ) {
	  ($skey,$uid,$to,$cip) = 
	      (shift(@command),shift(@command),shift(@command),shift(@command));
	  unless ( defined($skey) && defined($uid) &&
		   defined($to) && defined($cip) ) {
	      $reply = 'ERR PUT: not enough arguments';
	      last OP;
	  }
	  if ( @command ) {
	      $reply = 'ERR PUT: too many arguments';
	      last OP;
	  }
	  if ( exists($db->{$skey}) ) {
	      $reply = 'ERR PUT: mismatch';
	      syslog( 'alert','PUT spoof? uid:'.$db->{$skey}[1].
		      ' cip:'.$db->{$skey}[3].' last activity: '.
		      (time - $dh->get_atime($skey)) );
	      last OP;
	  }
	  $db->{$skey} = [$skey, $uid, $to, $cip];
	  if ( !defined($db->{$skey}) ) {
	      $reply = 'ERR Couldn\'t add';
	      $KILLED = 1;
	      last OP;
	  }
	  $dh->set_timeout($to,$skey)
	      if $to;
	  do_ats_login($uid);
	  $reply = 'OK ';
      }
      
      # DEL skey
      elsif ( $operation eq 'DEL' ) {
	  $skey = shift(@command);      
	  unless ( $skey ) {
	      $reply = 'ERR DEL: not enough arguments';
	      last OP;
	  }
	  if ( @command ) {
	      $reply = 'ERR DEL: too many arguments';
	      last OP;
	  }
	  if ( exists $db->{$skey} ) {
	      do_ats_logout($db->{$skey}[1]);
	      delete $db->{$skey};
	      $reply = 'OK';
	  } else {
	      $reply = "ERR $skey not found";
	  }
      }
      
      # { ATH | AUTH | AUT } skey clientip
      elsif ( $operation eq 'ATH' || 
	      $operation eq 'AUTH' || 
	      $operation eq 'AUT' ) {
	  ($skey,$cip) = (shift(@command),shift(@command));
	  unless ( defined($skey) && defined($cip) ) {
	      $reply = 'ERR ATH: not enough arguments';
	      last OP;
	  }
	  if ( @command ) {
	      $reply = 'ERR ATH: too many arguments';
	      last OP;
	  }
	  $Aryref = exists($db->{$skey}) ? $db->{$skey} : undef;
	  # FIXME: disabling ip checking as workaround for windoze's
	  #   broken dns round-robin client implementation
	  #if ( defined($Aryref) && $Aryref->[3] eq $cip ) {
	  if ( defined($Aryref) ) {
	      $reply = "YES $Aryref->[1]";
	  } else {
	      $reply = 'NO';
	  }
      }
      
      # PING
      elsif ( $operation eq 'PING' ) {
	  $reply = 'PONG';
      }
      
      # EVAL
      elsif ( $operation eq 'EVAL' ) {
	  $reply = 'OK ' . eval "return $buf";
	  
      }
      
      # QUIT
      elsif ( $operation eq 'QUIT' ) {
	  $reply = 'OK';
	  $KILLED = 1;
      }
      
      # menospeakie
      else {
	  $reply = "ERR unknown command: $operation "
	      . join(' ',@command);
      }
      
  } # OP
    
    $reply = 'ERR INT no reply defined'
	unless defined($reply);
    syslog('debug',"Exiting process_command: $reply");
    return "$reply\n";
} # sub process_command
