#!/webroot/perl/bin/perl -w
# $Id: GMA.pl,v 1.1.1.1 2000/09/03 03:39:58 santana Exp $
#
# 01.00.mario
#
# GMA.pl (Group Management Authority)
# Group and pseudo-group management cgi for macs
#

use CGI;
# send errors back to the browser and to the log
use CGI::Carp qw(fatalsToBrowser);
# love that database
use DBI;
# your window to the outside
use IO::Socket;
# sequential numbering to prevent duplicate tickets
use File::CounterFile;
# to read the html files
use IO::File;
# prevent mangling of cgi parameters
use URI::Escape;
# you're not paranoid, everyone really is out to get you
use strict;

#
# Get configured
use Config::IniFiles;
my($cfg,%cfg);
die("Couldn't configure!")
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => "../macs.cfg",
		      -default => "default");
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

# the Groups page
use constant GROUPSPAGE => "GMA_groups.html";
# the Members page
use constant MEMBERSPAGE => "GMA_members.html";
# the Details pages
use constant DETAILSPAGE => "GMA_details.html";
use constant NODETAILSPAGE => "GMA_nodetails.html";

# the main block
MAIN: {
  # get the CGI parameters
  my($query) = new CGI();
  # get the extra path information
  my($page) = $query->path_info();
  # check the extra path information to see which page to display
  if ( $page =~ m/groups/ ) {
    # go to the Groups page
    &groups($query);
  } elsif ( $page =~ m/members/ ) {
    # go to the Members page
    &members($query);
  } elsif ( $page =~ m/details/ ) {
    # go to the Details page
    &details($query);
  } else {
    # go to the autoregistration page by default
    &groups($query);
  }
  exit(0);
} # MAIN

# generate/handle the group management page
sub groups {
  my($query,$error) = (shift(),shift());

  # Do we need to do anything before generating the page?
  my($action) = $query->param("action");
  if ( $action eq "Modify" ) {
    # Manage group membership
    return members($query);
  } elsif ( $action eq "Delete" ) {
    # Delete a group
    $error .= delete_group($query);
  } elsif ( $action eq "New" ) {
    # Create a group
    $error .= add_group($query);
  }

  # Generate the page
  my(@groups) = get_groups();
  my($form_action) = $query->url() . "/groups";
  my($file) = new IO::File(GROUPSPAGE);
  while ( <$file> ) {
    s/\#\#ERROR\#\#/$error/go;
    s/\#\#ACTION\#\#/$form_action/go;
    s/\#\#GROUPLIST\#\#/"<option>".join("<option>",@groups)/goe;
    print;
  }
  $file->close;

  return;
} # sub groups

# Delete a group from the database
# return errors as string ("" for no error)
sub delete_group {
  return "<hr><strong>The Delete Group function " .
    "has not been implemented.</strong><hr>\n";

  my($query,$dbh) = (shift(),shift());
  my($error) = "";

  # Make sure group is empty (including waiting list)
  my($sth,$members);
  $sth = $dbh->prepare($cfg->{sql}{get_members});
  $sth->bind_param(1, $query->param("group"));
  $sth->execute();
  $members = $sth->fetchall_aryref;
  $sth->finish;
  return "<hr>Cannot delete group with members in it.<p>\n"
    if @$members;

  $sth = $dbh->prepare($cfg->{sql}{get_waitlist});
  $sth->bind_param(1, $query->param("group"));
  $sth->execute();
  $members = $sth->fetchall_aryref;
  $sth->finish;
  return "<hr>Cannot delete group with a waiting list.<p>\n"
    if @$members;

  # Do the dew
  # TODO: should I delete the row or set status = 0?
  $dbh->execute("");

} # sub delete_group

# Add a new group to the database
# return errors as string ("" for no error)
sub add_group {
  my($query) = shift();
  return "<hr><strong>The Add New Group function " .
    "has not been implemented.</strong><hr>\n";
} # sub add_group

# Return an array of all group names
sub get_groups {
  return ("The Group List function has not been implemented");
} # sub get_groups

# generate the group membership management page
sub members {
  my($query,$error) = (shift(),shift());

  # Do we need to do anything before generating the page?
  my($action) = $query->param("action");
  if ( $action eq "Details" ) {
    # Display member details
    # This is disabled for now -- see below
    #return details($query);
    $error .= "<hr>The Member Details function has not been implemented.<p>\n";
  } elsif ( $action eq "Delete" ) {
    # Delete a member
    $error .= delete_member($query);
  } elsif ( $action eq "Import" ) {
    # Upload a csv file of members
    return import_members($query);
  } elsif ( $action eq "Export" ) {
    # Download a csv file of members
    return export_members($query);
  } elsif ( $action eq "New" ) {
    # Create a group
    $error .= add_member($query);
  }

  # Generate the page
  my($group) = $query->param("group");
  my(@members) = get_members($group);
  my($form_action) = $query->url() . "/members";
  my($file) = new IO::File(MEMBERSPAGE);
  while ( <$file> ) {
    s/\#\#ERROR\#\#/$error/go;
    s/\#\#GROUP\#\#/$group/go;
    s/\#\#ACTION\#\#/$form_action/go;
    s/\#\#MEMBERSLIST\#\#/"<option>".join("<option>",@members)/goe;
    print;
  }
  $file->close;

  return;
} # sub members

# Delete a member from a group
# return errors as string ("" for no error)
sub delete_member {
  my($query) = shift();
  return "<hr><strong>The Delete Member function " .
    "has not been implemented.</strong><hr>\n";
} # sub delete_member

# Add a new member to a group
# return errors as string ("" for no error)
sub add_member {
  my($query) = shift();
  return "<hr><strong>The Add New Member function " .
    "has not been implemented.</strong><hr>\n";
} # sub add_member

# Return an array of all member names in a group
sub get_members {
  return ("The Member List function has not been implemented");
} # sub get_members

# Import members from an uploaded csv file
sub import_members {
  print "The Import function has not been implemented";
} # sub import_members

# Export members to a csv file for download
sub export_members {
  print "The Export function has not been implemented";
} # sub export_members

######
# The functionality below is disabled for now -- too
#   complicated by the fact that email is not unique
######

# generate the user details page
sub details {
  my($query,$error) = (shift(),shift());

  # Do we need to do anything before generating the page?
  my($action) = $query->param("action");
  if ( $action eq "Done"  || $action eq "No" ) {
    # Return to group membership page
    return members($query);
  } elsif ( $action eq "Yes" ) {
    # Delete a group
    $error .= mail_member($query);
  }

  # Generate the page
  my(@details) = get_details();
  $error .= mail_member_form()
    unless @details;
  my($form_action) = $query->url() . "/details";
  my($file) = new IO::File(DETAILSPAGE);
  while ( <$file> ) {
    s/\#\#ERROR\#\#/$error/go;
    s/\#\#ACTION\#\#/$form_action/go;
    s/\#\#\#\#/$details[0]/go
    s/\#\#LAST\#\#/$details[1]/go
    s/\#\#LOGIN\#\#/$details[2]/go
    s/\#\#EMAIL\#\#/$details[3]/go
    print;
  }
  $file->close;

  return;
} # sub details

# Send email to a non-existent member
sub mail_member {
  return "The Mail Member functionality has not been implemented";
} # sub mail_member

# Return html for send email to a non-existent member form
sub mail_member_form {
  my($query) = shift();
  return '<hr>This member has not registered. Would you like to send ' .
    'them email about their membership?<br>' .
    '<form method="POST" action="' . $query->url() . '/details">' .
    '<input type="submit" name="action" value="Yes">' .
    '<input type="submit" name="action" value="No">' .
    '</form><hr><p>';
} # sub mail_member_form

# Return an array of member details
sub get_details {
  return ("No First Name","No Last Name","No Login","No Email");
} # sub get_details

