#!/usr/bin/perl -w
# $Id: secure.pl,v 1.2 2002/03/25 04:11:32 santana Exp $
use strict;

use CGI;
use CGI::Carp;
use IO::Socket;
use IO::File;
use URI::Escape;

# the html pages
use constant HTMLPAGE => 'secure.html';
use constant ERRORPAGE => 'error.html';

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die("Couldn't configure: $!\n")
    unless $cfg->Sections();
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}
my($ups_port) = $cfg{UPS}{s};
$ups_port =~ s/^.*:([0-9]+)$/$1/;


my($query) = new CGI();
my($page) = $query->path_info();
return bad_page($query,'No CGI object!')
    unless ref($query) =~ '^CGI';
my($uidhdr) = uc($cfg{AUS}{http_uid_header}); $uidhdr =~ tr/-/_/;
my($uid) = $ENV{"HTTP_$uidhdr"};
my($login) = $ENV{HTTP_MACS_LOGIN};
my($name) = $ENV{HTTP_MACS_NAME};
my($welcome,$profile);
if ( defined($login) && defined($name) ) {
    $welcome = $query->h1("Welcome, $login") . "Hello $name my only friend. Choose an application to try to visit.<br>";
    $profile = get_profile();
} elsif ( defined($uid) ) {
    $welcome = $query->h1("Welcome, user $uid") . 'You are logged in.';
    $profile = 'Something\'s a little wrong -- I don\'t have access to your profile.';
} else {
    $welcome = $query->h1('Welcome, stranger') . 'I don\'t know you, but pick an application you might like to log into.';
    $profile = 'If you log in, I will show you your profile.';
}
# open the html file
my($htmlfile) = new IO::File('<' . HTMLPAGE);
return bad_page($query,'Error opening '.HTMLPAGE)
    unless defined($htmlfile) && $htmlfile;
# print the CGI headers
print $query->header();
# print the html file substituting the keywords
while( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#WELCOME\#\#/$welcome/g;
    $line =~ s/\#\#PROFILE\#\#/$profile/g;
    print $line;
}
# close the html file
$htmlfile->close();

# the error page 
sub bad_page {
    my($query,$error) = (shift(),shift());
    my($htmlfile) = new IO::File('<' . ERRORPAGE);
    die "Could not find error page"
	unless defined($htmlfile);
    print $query->header()
	if defined($query) && ref($query) =~ /^CGI/;
    while ( my $line = $htmlfile->getline() ) {
	$line =~ s/\#\#ERROR\#\#/$error/g;
	print $line;
    }
    $htmlfile->close();

    return 0;
} # sub bad_page

sub get_profile {
    my($p) = '<ul>';

#    $p .= '<li>need: HTTP_'.uc($uidhdr)."</li>\n";
#    $p .= "<li>need: HTTP_MACS_LOGIN</li>\n";
#    $p .= "<li>need: HTTP_MACS_NAME</li>\n";
#    for ( keys %ENV ) {
#	$p .= "<li>$_: $ENV{$_}</li>\n";
#    }
#    return $p.'</ul>';

    my($u) = new IO::Socket::INET( PeerAddr => $cfg{UPS}{s} );
    return bad_page($query,"Couldn't connect to UPS: $!")
	unless defined($u) && $u->peerport() == $ups_port;
    $u->print("LIST 1 $uid\n");
    my($items) = $u->getline();
    return "Error fetching core profile list: $items"
	unless $items =~ s/^\s*YES\s+//i;
    return 'No core profile for you.'
	unless $items =~ /\S/;
    my($len);
    for ( split(' ',$items) ) {
	$p .= "<li>$_: ";
	$u->print("GET 1 $uid $_\n");
	$len = $u->getline();
	return "Error fetching core item $_: $len"
	    unless $len =~ s/^\s*YES\s+([0-9]+)\s*$/$1/i;
	$u->read($p,$len,length($p));
	$p .= "</li>\n";
    }
    $p .= '</ul>';
    return $p;
} # sub get_profile
