#!/usr/bin/perl -w
# $Id: welcome.pl,v 1.2 2002/03/25 04:11:32 santana Exp $
use strict;


use CGI;
use CGI::Carp;
use IO::File;
use IO::Socket;
use URI::Escape;

use constant HTMLPAGE => 'welcome.html';
use constant ERRORPAGE => 'error.html';


#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die("Couldn't configure: $!\n")
    unless $cfg->Sections();
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}
my($ups_port) = $cfg{UPS}{s};
$ups_port =~ s/^.*:([0-9]+)$/$1/;
my($uidhdr) = uc($cfg{AUS}{http_uid_header}); $uidhdr =~ tr/-/_/;
my($uid) = $ENV{"HTTP_$uidhdr"};
my($profile) = "";

my($query) = new CGI();

return bad_page($query,'No CGI object!')
    unless ref($query) =~ '^CGI';

# create the drop-down of apps
my($apps);# = get_apptable($query);
my($login) = $ENV{HTTP_MACS_LOGIN};
my($name) = $ENV{HTTP_MACS_NAME};
my($welcome);
if ( defined($login) && defined($name) ) {
    $welcome = $query->h1("Welcome, $login") . "Hello $name my only friend. Choose an application to try to visit.<br>";
    $profile = get_profile();
} elsif ( defined($uid) ) {
    $welcome = $query->h1("Welcome, user $uid") . 'You are logged in, now pick an application.';
} else {
    $welcome = $query->h1('Welcome, stranger') . 'I don\'t know you, but pick an application you might like to log into.';
}
# open the html file
my($htmlfile) = new IO::File('<' . HTMLPAGE);
return bad_page($query,'Error opening '.HTMLPAGE)
    unless defined($htmlfile) && $htmlfile;
# print the CGI headers
print $query->header();
# print the html file substituting the keywords
while( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#WELCOME\#\#/$welcome/g;
    $line =~ s/\#\#APPS\#\#/$apps/g;
    $line =~ s/\#\#PROFILE\#\#/$profile/g;
    print $line;
}
# close the html file
$htmlfile->close();

# the error page 
sub bad_page {
    my($query,$error) = (shift(),shift());
    my($htmlfile) = new IO::File('<' . ERRORPAGE);
    die "Could not find error page"
	unless defined($htmlfile);
    print $query->header()
	if defined($query) && ref($query) =~ /^CGI/;
    while ( my $line = $htmlfile->getline() ) {
	$line =~ s/\#\#ERROR\#\#/$error/g;
	print $line;
    }
    $htmlfile->close();

    return 0;
} # sub bad_page

# create the drop-down of applications
sub get_apps {
    my($query) = shift();
    my($cmeth);
    # open a connection to the LMS
    my($lms);
    croak("Couldn't connect to LMS: $!\n");
#	unless $lms = new IO::Socket::INET( PeerAddr => $cfg{LMS}{s} );
    croak("Couldn't connect to LMS: $!");
#	unless defined($lms) && $lms->peerport == $lms_port;
    $lms->print("METH\n");
    my($meths) = $lms->getline;
    $lms->close;
    # figure out default method
    my($defmeth) = 'internal';
    #if ($query->remote_addr() =~ /^192\.168/) {
    #  $defmeth = 'Yourco';
    #} else {
    #  $defmeth = 'internal';
    #}
    if ( $meths !~ s/^\s*OK\s*// || !$meths ) {
	# if we can't get a list, make one w/ just the defmethod
	$cmeth = "Unable to get method list, using $defmeth<br>\n" .
	    "<input type=hidden name=method value=\"$defmeth\">\n";
    } else {
	$cmeth = '<select name=method size=1>';
	foreach my $meth ( split(' ',$meths) ) {
	    $cmeth .= '<option';
	    $cmeth .= ' selected'
		if $cmeth eq $defmeth;
	    $cmeth .= ">$meth\n";
	}
	$cmeth .= "\n</select>\n";
    }
    return $cmeth;
} # sub get_methodtable


sub get_profile {
    my($p) = '<ul>';

    $p .= "<li>need: HTTP_$uidhdr</li>\n";
    $p .= "<li>need: HTTP_MACS_LOGIN</li>\n";
    $p .= "<li>need: HTTP_MACS_NAME</li>\n";
    for ( keys %ENV ) {
	$p .= "<li>$_: $ENV{$_}</li>\n";
    }
    $p .= '</ul><hr>';
    if ( $uid ) {
	$p .= '<ul>';
    } else {
	return $p;
    }

    my($u) = new IO::Socket::INET( PeerAddr => $cfg{UPS}{s} );
    return bad_page($query,"Couldn't connect to UPS: $!")
	unless defined($u) && $u->peerport() == $ups_port;
    $u->print("LIST 1 $uid\n");
    my($items) = $u->getline();
    return "Error fetching core profile list: $items"
	unless $items =~ s/^\s*YES\s+//i;
    return 'No core profile for you.'
	unless $items =~ /\S/;
    my($len);
    for ( split(' ',$items) ) {
	$p .= "<li>$_: ";
	$u->print("GET 1 $uid $_\n");
	$len = $u->getline();
	return "Error fetching core item $_: $len"
	    unless $len =~ s/^\s*YES\s+([0-9]+)\s*$/$1/i;
	$u->read($p,$len,length($p));
	$p .= "</li>\n";
    }
    $p .= '</ul>';
    return $p;
} # sub get_profile
