#!/usr/bin/perl -w
# $Id: LIA.pl,v 1.6 2002/04/16 21:55:09 santana Exp $ 
use strict;

# 04/26/99 Blake Mitchell
# 11.8.99.mario
#
# login screen CGI for macs


use CGI;
use CGI::Carp;
use IO::Socket;
use IO::File;
use URI::Escape;

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die("Couldn't configure: $!\n")
    unless $cfg->Sections();
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}

# the html pages
use constant LOGINPAGE => 'LIA.html';
use constant ERRORPAGE => 'LIA_error.html';

# find out which page to display based on extra path info
MAIN: {
    my($query) = new CGI();
    my($page) = $query->path_info();
    # go to the login form by default
    &login_page($query) unless defined($page);
    if($page =~ m/login/) {
	# the login form
	&login_page($query);
    } elsif($page =~ m/lresult/) {
	# the login processor
	&lresult_page($query);
    } else {
	# go to the login form by default
	&login_page($query);
    }
    exit(0);
} # MAIN

# generate the login page
#
# inputs: CGI object param
#         error string (optional param)
#         url or server/uri[/port] POST or GET vars
#         ura POST or GET var
# output: login form
# process: replaces some keywords in the LOGINPAGE
#   with values gleaned from the inputs
sub login_page {
    my($query,$error) = (shift(),shift());
    $error = ''
	if !defined($error);
    return bad_page($query,'No CGI object!')
	unless ref($query) =~ '^CGI';

    # create the url to post the form to
    my($actionurl) = $query->url() . '/lresult';
    # get the start url to redirect back to
    my($starturl) = get_start($query);
    # create the drop-down of methods
    my($cmeth) = get_methodtable($query);
    # are we told to display the URA?
    my($ura) = get_ura($query, $starturl);
    # do we have any default info?
    my($username) = ($query->param("username") || '');
    my($passwd) = ($query->param("passwd") || '');
    my($hidden) = ($query->param("ura") || '');
    $hidden = "<input type=hidden name=ura value=".$hidden.">"
	if $hidden;

    # open the html file
    my($htmlfile) = new IO::File('<' . LOGINPAGE);
    return bad_page($query,'Error opening '.LOGINPAGE)
	unless defined($htmlfile) && $htmlfile;
    # print the CGI headers
    print $query->header();
    # print the html file substituting the keywords
    while( my $line = $htmlfile->getline() ) {
	$line =~ s/\#\#ACTION\#\#/$actionurl/g;
	$line =~ s/\#\#USERNAME\#\#/$username/g;
	$line =~ s/\#\#PASSWD\#\#/$passwd/g;
	$line =~ s/\#\#START\#\#/$starturl/g;
	$line =~ s/\#\#METHODS\#\#/$cmeth/g;
	$line =~ s/\#\#HIDDEN\#\#/$hidden/g;
	$line =~ s/\#\#ERROR\#\#/$error/g;
	$line =~ s/\#\#URA\#\#/$ura/g;
	print $line;
    }
    # close the html file
    $htmlfile->close();
    return 1;
} # login_page

# this checkes the user provided login data
sub lresult_page {
    my($query) = shift();
    my($username, $passwd, $defmeth, $method);

    # give an error if no username was provided
    unless (defined $query->param('username') and
	    $username = $query->param('username')) {
	return login_page($query,'No Username');
    } 
    # whine if there's an illegal character in it
    if ( $username =~ s/([^A-Za-z0-9._-]+)/<bold>$1<\/bold>/g ) {
	$query->param(-name=>'username',-value=>'');
	return login_page($query,"Bad characters in $username");
    }
    $username = uri_escape($username);

    # give an error if no password was provided
    unless( defined $query->param('passwd') and
	    $passwd = $query->param('passwd')) {
	return login_page($query,'No Password');
    }
    $passwd = uri_escape($passwd);

    # give an error if no method was provided
    unless ( defined $query->param('method') and
	     $method = $query->param('method') ) {
	return bad_page($query,'No Login Method');
    }

    # open a connection to the LMS
    my($lms);
    $lms = new IO::Socket::INET( PeerAddr => $cfg{LMS}{s} );
    return bad_page($query,"Couldn't connect to LMS: $!")
	unless defined($lms) && $lms->peerport();

    # dew the doo
    $lms->print("AUTH $method $username $passwd ",
		$query->remote_addr(),' ',$lms->sockport(),"\n");
    my($rv) = $lms->getline(); chomp($rv);
    # (BAM) fixed broken ifs
    if ( !defined($rv) ) {
	return bad_page($query,'No response from LMS');
    } elsif($rv =~ m/^YES\s+(\S+)\s*(.*)$/i) {
	return redirect_cookie($query, $1);
    } elsif($rv =~ m/^NO\s+(.*)$/i) {
	return login_page($query,'Authentication denied!');
    } else {
	return bad_page($query,"Login Method Server failed: $rv");
    }
} # sub lresult_page

# feed the browser a cookie and redirect it to the start url
sub redirect_cookie {
    my($query,$skey) = (shift(),shift());
    my($start) = get_start($query);
    my($redir);

    if ( defined($query->param('change_passwd')) &&
	 $query->param('change_passwd') ) {
	$redir = $cfg{'url cgi'}{PCA} . '?url=' . $start;
    } elsif ( defined($redir) && $redir ) {
	$redir = $redir . "?url=" . $start;
    } else {
	$redir = uri_unescape($start);
    }

    # format the cookie
    my($authcookie) = $query->cookie(-name   => $cfg{LMS}{cookie_name}, 
				     -value  => $skey, 
				     -path   => '/', 
				     -domain => $cfg{LMS}{cookie_domain});
    # set the cookie
    print("Set-cookie: $authcookie\n");
    # send the redirect headers
    print("Status: 302 Moved\n");
    # with the start url
    print("Location: $redir\n\n");

    return 0;
} # sub redirect_cookie

# the error page 
sub bad_page {
    my($query,$error) = (shift(),shift());
    my($htmlfile) = new IO::File('<' . ERRORPAGE);
    die "Could not find error page"
	unless defined($htmlfile);
    print $query->header()
	if defined($query) && ref($query) =~ /^CGI/;
    while ( my $line = $htmlfile->getline() ) {
	$line =~ s/\#\#ERROR\#\#/$error/g;
	print $line;
    }
    $htmlfile->close();

    return 0;
} # sub bad_page

# construct the url to redirect back to after login, based on
# information posted by the web server that redirected here
sub get_start {
    my($query) = shift();
    # get the server name and uri (path)
    my($starturl);
    if ( defined($query->param('url')) ) {
	$starturl = $query->param('url');
    } elsif(defined $query->param('server') and defined $query->param('uri')) {
	##FIXME make sure 'server' and 'uri' are not used & remove this
	# if the port is not 80
	if(defined $query->param('port') and $query->param('port') != 80) {
	    # add it to the server name
	    $query->param('server', $query->param('server') . ':' . 
			  $query->param('port'));
	}
	# create the url from the server name and uri
	$starturl = 'http://' . $query->param('server') . 
	    $query->param('uri');
    } else {
	# if there's no info, use the default url
	$starturl = $cfg{'url misc'}{LOGGED_IN};
    }
    return $starturl;
} # sub get_start

# create the drop-down of methods
sub get_methodtable {
    my($query) = shift();
    my($cmeth);
    # open a connection to the LMS
    my($lms);
    croak("Couldn't connect to LMS: $!\n")
	unless $lms = new IO::Socket::INET( PeerAddr => $cfg{LMS}{s} );
    croak("Couldn't connect to LMS: $!")
	unless defined($lms) && $lms->peerport;
    $lms->print("METH\n");
    my($meths) = $lms->getline;
    $lms->close;
    # figure out default method
    my($defmeth) = 'internal';
    #if ($query->remote_addr() =~ /^192\.168/) {
    #  $defmeth = 'Yourco';
    #} else {
    #  $defmeth = 'internal';
    #}
    if ( $meths !~ s/^\s*OK\s*// || !$meths ) {
	# if we can't get a list, make one w/ just the defmethod
	$cmeth = "Unable to get method list, using $defmeth<br>\n" .
	    "<input type=hidden name=method value=\"$defmeth\">\n";
    } else {
	$cmeth = '<select name=method size=1>';
	foreach my $meth ( split(' ',$meths) ) {
	    $cmeth .= '<option';
	    $cmeth .= ' selected'
		if $cmeth eq $defmeth;
	    $cmeth .= ">$meth\n";
	}
	$cmeth .= "\n</select>\n";
    }
    return $cmeth;
} # sub get_methodtable

# should we link to the ura? default is no
sub get_ura {
    my($query) = shift();
    my($starturl) = shift();
    my($ura);
    if ( defined($query->param('ura')) && $query->param('ura') ) {
	$ura =
	    '<form method=POST name=URAForm action="'.$cfg{'url cgi'}{URA}.'">
            <input type=hidden name=url value="' . $starturl . '">
            <input type=submit value="New Users">
            </form>';
    } else {
	$ura = "";
    }
    return $ura;
} # sub get_ura
