#!/opt/perl/bin/perl -w
# $Id: LMC_NT.pl,v 1.2 2001/08/07 00:59:26 santana Exp $
#
# LMCd - Login Method Client
#
# An implementation of the LMC component of macs
# 
# 10.99.mario
#

package LMCd;
use strict;
use lib "../lib";
use Getopt::Std;
use IO::Socket;
use Authen::Smb;

#
# the command line
# this will leave "inifile" in @ARGV
my(%opts) = ( v => 0 );
getopts('vfh',\%opts);

die("
Usage: $0 [ -h ] [ -v ] [ -f ] [ cfgfile ]
 	h prints a synopsis (what you're reading),
	v makes me very chatty,
	f keeps me in the foreground, and
	cfgfile is the macs config file (defaults to ../macs.cfg).

Note that this has changed from -h host -p port.
Also note that cfgfile has to be the last option on the line.

")
  if $opts{h};

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || "../macs.cfg");
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => "default" );
die("Couldn't configure: $!\n")
    unless $cfg->Sections(); 
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

$SIG{__WARN__} = sub { ;; }
    unless $opts{v};

#
# the lms
my($lms) = new IO::Socket::INET(PeerAddr => $cfg{LMS}{s},
                           Reuse    => 1);
die("Could not connect to LMS at $cfg{LMS}{s} -- $!\n")
    unless $lms->peername;
warn("Connected to LMS at $cfg{LMS}{s}\n");

#
# Register
eval {
  die("Trouble writing to socket: $!\n")
    unless $lms->{sock}->print("REG NT\n");
  die("Trouble reading from socket: $!\n")
    unless my($rv) = $lms->{sock}->getline();
  chomp($rv);
  die("Error returned from LMS: $rv\n")
    unless $rv eq "OK";
};
die("Could not register with LMS at $cfg{LMS}{s} -- $@\n")
  if $@;
warn("Registered with LMS as method NT\n")
  if $opts{v};

# make sure we're running in the background
FORK: do {
  # the process id variable
  my($pid);
  # spawn a child
  if($pid = fork()) {
    # parent process here, bye bye
    exit(0);
  } elsif(defined $pid) {
    # child process here, we continue
    last FORK;
  } elsif($! =~ m/[Nn]o more process/) {
    # can't fork, sleep and try again
    sleep(5);
    redo FORK;
  } else {
    # unrecoverable fork error
    die("Couldn't fork: $!\n");
  }
} unless %opts{f};

#
# Now just sit on the socket
my($req,$login,$passwd,$nothing,$ar);
while ( $req = $lms->{sock}->getline() ) {
  # Check for good command/method
  if ( $req =~ s/^\s*INFO\s*//i ) {
    do_info($req);
  } else if ( $req =~ s/^\s*AUTH\s*//i ) {
    do_auth($req);
  } else {
    $lms->{sock}->print("ERR Unknown command: $req\n");
    warn("ERR Unknown command: $req\n")
	if $opts{v};
  }
} # while getline (from LMS)

# FIXME: this should get implemented
sub do_info {
  my($req) = shift();
  $lms->{sock}->print("NO $ar\n");
} # sub do_info

sub do_auth {
  my($req) = shift();
  if ( $req !~ s/^\s*NT\s*//i ) {
    $lms->{sock}->print("ERR Unknown method: AUTH $req\n");
    warn("ERR Unknown method: AUTH $req\n")
      if $opts{v};
    next REQ;
  }
  # check for proper args
  ($login,$passwd,$nothing) = split(' ',$req,3);
  unless ( $login && !$nothing ) {
    $lms->{sock}->print("ERR Wrong number of args: AUTH NT $req\n");
    warn("ERR Wrong number of args: AUTH NT $req\n")
      if $opts{v};
    next REQ;
  }
  # do the doo
  $ar = Authen::Smb::authen($login,$passwd,PDC,BDC,DOMAIN);

  # Thwap!
  if ( $ar != 0 ) {
    $lms->{sock}->print("NO $ar\n");
    warn("Authentication failed with $ar\n")
      if $opts{v};
    next REQ;
  }
  # woo-hoo
  warn("Authentication passed\n")
    if $opts{v};
  $lms->{sock}->print("YES\n");
} # sub do_auth 

# Hmm... let's retry after a short pause
warn("Left main loop.. reinitializing\n")
  if $opts{v};
sleep(10);

# the LMS connection
$lms->close;
unless ( $lms = new IO::Socket::INET(PeerAddr => $cfg{LMS}{s},
                                Reuse    => 1) ) {
  unlink $pidf;
  die("Could not reconnect to LMS at $cfg{LMS}{s} -- $!\n");
}
       
eval {
  die("Trouble writing to socket: $!\n")
    unless $lms->{sock}->print("REG NT\n");
  die("Trouble reading from socket: $!\n")
    unless my($rv) = $lms->{sock}->getline();
  chomp($rv);
  die("Error returned from LMS: $rv\n")
    unless $rv eq "OK";
};
die("Could not register with LMS at $cfg{LMS}{s} -- $@\n")
  if $@;

# and sit on the socket again
warn("Reentering main loop\n")
  if $opts{v};
goto REQ;
