$Id: README.login,v 1.2 2001/08/07 01:04:47 santana Exp $

This document will try to explain the macs login scheme. It's very
straighforward for the usual cases, but there's some complexity in
dealing with exceptional cases.

First, a relevant tla dictionary.

LIA - LogIn Authority
	The piece that requests a login on behalf of a user. This can
	be a CGI, it can be embedded into another application that's
	been macs'd, or whatever.
LMS - Login Method Server
	Processes login requests. Doesn't do the actual
	authentication, and usually doesn't do much more than
	distribute the request to the correct LMC; however, it
	contains the logic for exceptional cases and so is the
	trickiest piece in the puzzle.
LMC - Login Method Client
	This piece actually does the authentication. An LMC registers
	a login method with an LMS. The LMS hands off login requests
	of that method to the LMC which does a database lookup, or an
	NT call, or consults the magic eight ball. One LMC can
	register multiple methods, but usually an LMC is written for
	each login method as they are added to the macs installation.
AUS - AUthentication Server
	When a login request is successful, a session is stored with
	the AUS.
AUC - AUthentication Client
	The piece that makes sure a user is logged into macs, by
	checking with the AUS. This can be a web server plugin, it can
	be embedded into another application that's been macs'd, or
	whatever.

Although there's a built-in authentication mechanism, macs' login
scheme is designed to live on top of other authentication schemes. You
can write (or use already written) LMC daemons to let your people log
in to macs using logins/passwords from NT, or unix, or SAP, or some
proprietary system written in COBOL that's been in continuous
production since before blood sacrifices went out of style. There is
one login method for each authentication mechanism.

Let's recap the above as a picture. Then we'll get into the details.



1.user enters  3.The LMS picks an
login,pwd &    LMC and sends it
method         the login/passwd 
                        
o o ====> LIA ====> LMS ====> LMC 
 U                   |
     2.LIA passes    |    4.LMC returns yeay
     this to LMS    \|/   or nay to LMS
                     V
                    AUS

        5.The LMS inits a session for the 
        logged-in user and returns yeay to 
        the LIA, which does an appropriate
        things, (e.g., redirect the user to
        where they were trying to go when
        they needed to log in.)


OK, so I'm no ASCII artist. Come over sometime and I'll show you the
oil on canvas version. ;-)

/*************
 ** Details **
 *************/

The protocols are all text, so you can poke around with telnet or
nc. Of the protocols represented (LIA/LMS, LMS/LMC, and LMS/AUS) only
the LMS/LMC one has any initialization (the LMC must register the
method(s) it can handle.) This means there is little price to pay for
opening a new connection for each request. This will change, of
course, once ssl is in place, if it's being used.


So, the logic goes:
-------------------

LIA says to LMS, "Is <login>/<passwd> a valid <method> login?"

LMS looks for an LMC that has registered <method>. If it doesn't find
one, it replies to LIA, "No", and LIA does some approprate things,
like tell the user, etc., and that's that. OTOH, if an LMC for
<method> has registered with LMS, LMS says to that LMC, "Is
<login>/<passwd> a valid combo?"

LMC does the appropriate lookup. If <login>/<passwd> are invalid, LMC
replies to LMS, "No", and LMS replies to LIA, "No", and that's
that. OTOH, if <login>/<passwd> match up, LMC replies to LMS, "Yes".

LMS demaps the login into a username. (More on this later. Suffice to
say that the method-specific login is turned into a "canonical" name
by which macs knows this user.) LMS says to AUS, "Hey, <username> just
logged in." If there is any problem AUS replies to LMS, "So?", and LMS
says to LIA, "Hey, there's a problem here" and LIA does something
appropriate, like display an error message to the user. OTOH, if the
AUS can create a session, it tags the session with a key, and replies
to LMS, "Cool, here's the <sessionkey>"

LMS replies to LIA, "Yes, and here's the <sessionkey>."

LIA does something appropriate like let the user continue accessing
the resource.

demapping
---------

1.  Joe Random Hacker might have several logins: jrh on unix, hackerjr
on NT, joerh on SAP, and hootinannie on the IRC server. Right now, he
might be logging into macs for the first time. He submits a
<login>/<passwd> pair for a particular method (say jrh/cantcrackthis
for unix) to LIA, which forwards it to LMS. LMS finds that this LMC
over here registered the unix method and asks her about
jrh/cantcrackthis. LMC says it's good. Now comes the demap.

2.  LMS looks for a username that's mapped to 'jrh on unix', but finds
that there's no match, because Joe's never used that login to get into
macs before. Actually, he's never used *any* login to get into macs
before.

3.  Now the LMS has to create a mapping for 'jrh on unix'. macs
usernames have several bits of information attached to them, like name
and email. LMS asks LMC for these data about 'jrh on unix', and looks
for existing macs users that have the same data. (Right now, LMS only
looks at email, but this needs to be made more flexible. Also, LMS
/should/ ask UPS, but that will come.) If it finds one and is
confident that they are the same person, it creates a mapping from
'jrh on unix' to that username. In our case, of course, it finds none,
because Joe's never logged into macs and therefore has no username
there yet.

4.  So LMS creates a macs username for Joe. It will try to use jrh but
will munge the username to make it unique. Let's say the username
becomes jrh123. LMS then makes a mapping to 'jrh123' from 'jrh on
unix'.

5.  Finally, LMS creates a session for Joe as jrh123.

Now the next time Joe logs into macs using jrh/cantcrackthis on unix,
LMS will find a username that's mapped to 'jrh on unix' in step 2
above and will skip steps 3 and 4.

But if Joe decides to use a <login>/<password> for a different method
(say hackerjr/easypwd for NT), LMS will find a username to map to in
step 3, because the LMC that registered to service NT logins tells LMS
that 'hackerjr on NT' has an email that matches what LMS already has
for jrh123. LMS makes a mapping to 'jrh123' from 'hackerjr on NT' and
creates a session for Joe as jrh123.






/***********************
 ** Old Documentation **
 ***********************/

The login process is at the heart of WACA. It is a necessarily
complex system of cgi scripts and servers. The components are:

-- the Login Method Server (LMS): services requests for
   authentication from an LIA (see below.) It dispatches the
   actual authentication work to an appropriate LMC (see below)
   and creates a WACA session with an AUS (see below) if the
   operation passes. It returns to the LIA initiating the request
   the results of the operation.

-- the Login Authorities (LIA): a cgi which presents to the user a
   request for authentication information (e.g., a login/password
   combination) and initiates the login process with an LMS. Based
   on the result of the operation, the user is asked to try again,
   or is redirected to the referring page (what the user was
   trying to access before they were redirected to the LIA) or to
   some alternate page at the LMS's request. Different versions of
   the LIA contain links to the URA (see below), to the RCA's
   password change page (see below), and/or to the LPA (see
   below.)

-- the Login Method Clients (LMC): handles authentication requests
   on behalf of the LMS. An LMC will register itself with an LMS
   for one or more authentication methods (e.g., NT-, WWW-, or
   SAP-based authentication) and will service requests
   accordingly. The LMC may determine that a user should be
   redirected to a particular url after logging in (e.g., to a
   change password page if their password has expired) and will
   tell the LMS to tell the LIA if this is the case.

-- the Authentication Server (AUS): stores active WACA sessions as
   created by the LMS. Serves authentication requests for the AUC
   (see below.) Handles session timeouts.

-- the Authentication Client (AUC): the link between WACA-enabled
   applications and the WACA authentication back-end. When access
   to a restricted area is requested, the AUC will check that the
   requester has an active and valid WACA session with the AUS
   before continuing. If not, the user is redirected to an LIA
   instead. The url for which access failed is passed to the LIA
   for later redirection.

-- the User Registration Agent (URA): a cgi which requests
   registration information from the user and creates a WACA
   account. The account will be authenticated via the WWW
   method. It will belong to no groups. The new user is logged
   into WACA and is redirected to the referring url.

-- the Registration Change Authority (RCA): a cgi which presents
   to the user a form with their current registration
   information. After making changes to one of the fields, the
   submit button for that field will effect the change and reload
   the page. Clicking done will redirect the user to the referring
   url. If called with appropriate parameters, will only display
   the password-change portion of the page.

-- the Lost Password Authority (LPA): The user is asked for
   various information about the account in question. Matching
   accounts are displayed, and the user selects one. The accounts
   password is reset to a random string and the string is sent to
   the email registered for the account. The account's
   password-change flag is set.
