#!/usr/bin/perl -w
#
# test_LMSd
#
# 2.99.BAM
# 7.99.mario
#

package test_LMSd;
use IO::Socket;
use Config::IniFiles;

#
# Get configured
my($cfg,%cfg);
die("Couldn't configure!")
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => '../macs.cfg');
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

#
# Register a fake LMC
$lmc = new IO::Socket::INET(PeerAddr => $cfg{LMS}{s},
			     Proto    => 'tcp');
if ( !$lmc ) {
  print "Error connecting to $cfg{LMS}{s}: $!\n";
  exit(1);
}
$lmc->autoflush(1);

if ( defined($ARGV[0]) && $ARGV[0] eq 'q' ) { 
  $writ = "QUIT\n";
  print 'Sent: (',$lmc->write($writ,length $writ),') ',$writ;
  print "Want: OK\n";
  print 'Resp: ',$lmc->getline(),"\n\n";
  exit;
}

$writ = "PING\n";
print "Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "Want: PONG\n";
print "Resp: ";
print $lmc->getline(),"\n\n";

$writ = "AUTH internal guest guest 9.9.9.9 5093\n";
print "Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "Want: YES login_id\n";
print "Resp: ";
print $lmc->getline(),"\n\n";



$writ = "REG test_LMSd\n";
print "Sent: (",$lmc->write($writ,length $writ),") ",$writ;
$writ = $lmc->getline();
print "Want: OK\n";
print "Resp: ",$writ,"\n\n";
if ( $writ =~ /^\s*NO\s?.*$/i ) {
    print "Error setting up LMC for method test_LMSd.\n";
    print "Maybe you need to do:\n";
    print "\tinsert into macs_login_method (name) values ('test_LMSd')\n\n";
    exit(1);
}


#
# This is our client
$lia = new IO::Socket::INET(PeerAddr => $cfg{LMS}{s},
			    Proto    => 'tcp');
if ( !$lia ) {
  print "Error connecting to $cfg{LMS}{s}: $!\n";
  exit(1);
}
$lia->autoflush(1);

$writ = "AUTH test_LMSd login password ip port\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "\t(lmc) Want: AUTH test_LMSd login password\n";
print "\t(lmc) Resp: ";
print $lmc->getline();
$writ = "NO Testing\n";
print "\t(lmc) Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "(lia) Want: NO Testing\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "AUTH test_LMSd login password ip port\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "\t(lmc) Want: AUTH test_LMSd login password\n";
print "\t(lmc) Resp: ";
print $lmc->getline();
$writ = "YES\n";
print "\t(lmc) Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "(lia) Want: ERR AUTH: bad IP address\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "AUTH test_LMSd login password 1.1.1.1 1\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "\t(lmc) Want: AUTH test_LMSd login password\n";
print "\t(lmc) Resp: ";
print $lmc->getline();
$writ = "YES testing NoSuch User\n";
print "\t(lmc) Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "(lia) Want: YES login_id\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "REG test_LMSd\n";
print "Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "Want: NO\n";
print "Resp: ";
print $lmc->getline(),"\n\n";

$writ = "REG test_LMSd_anothermethod\n";
print "Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "Resp: NO test_LMSd_anothermethod unknown method\n";
print "Resp: ";
print $lmc->getline(),"\n\n";

$writ = "AUTH test_LMSd login password ip port\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "\t(lmc) Want: AUTH test_LMSd login password ip port\n";
print "\t(lmc) Resp: ";
print $lmc->getline();
$writ = "NO Testing\n";
print "\t(lmc) Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "(lia) Want: NO Testing\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "METH\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "(lia) Want: OK test_LMSd\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "DEREG test_LMSd_anothermethod\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "(lia) Want: NO Method test_LMSd_anothermethod not registered\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "METH\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "(lia) Want: OK test_LMSd\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

$writ = "DEREG test_LMSd\n";
print "(lmc) Sent: (",$lmc->write($writ,length $writ),") ",$writ;
print "(lmc) Want: OK\n";
print "(lmc) Resp: ";
print $lmc->getline(),"\n\n";

$writ = "METH\n";
print "(lia) Sent: (",$lia->write($writ,length $writ),") ",$writ;
print "(lia) Want: OK\n";
print "(lia) Resp: ";
print $lia->getline(),"\n\n";

