#!/usr/bin/perl -w
# $Id: LOA.pl,v 1.4 2002/04/09 15:25:14 santana Exp $
use strict;
#
# LogOut Authority for MACS
#
use CGI;
use CGI::Carp;
use IO::Socket;
use IO::File;
use URI::Escape;


#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || '../macs.cfg');
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => 'default' );
die("Couldn't configure: $!\n")
    unless $cfg->Sections();
for ( keys %{$cfg{env}} ) {
    $ENV{$_} = $cfg{env}{$_};
}
my($lms_port) = $cfg{LMS}{s};
$lms_port =~ s/^.*:([0-9]+)$/$1/;




# the html pages
use constant ERRORPAGE => 'LOA_error.html';

my($query) = new CGI();
my($skey) = $query->cookie($cfg{LMS}{cookie_name});

return redirect_cookie($query)
    unless defined($skey) && $skey;

# open a connection to the AUS
my($aus);
$aus = new IO::Socket::INET(PeerAddr => $cfg{AUS}{s});
return bad_page($query,"Couldn't connect to AUS: $!")
    unless defined($aus) && $aus->peerport();

# dew the doo
$aus->print("DEL $skey\n");
my($rv) = $aus->getline();
# (BAM) fixed broken ifs
if ( $rv eq "OK\n" ) {
  redirect_cookie($query);
} elsif ($rv eq "ERR $skey not found\n") {
  redirect_cookie($query);
} elsif ($rv =~ m/^ERR\s+(.*)$/i) {
  bad_page($query,"Logout Error: $1");
} else {
  bad_page($query,"AUS Failure: $rv");
}

# empty the browser's cookie and redirect it to the start url
sub redirect_cookie {
  my($query) = shift();
  my($start) = get_start($query);

  # format the 
  my($authcookie) = $query->cookie(-name   => $cfg{LMS}{cookie_name},
				   -value  => '0',
				   -path   => '/',
				   -domain => $cfg{LMS}{cookie_domain});
  # set the cookie
  print("Set-cookie: $authcookie\n");
  # disable cache
  print("Pragma: no-cache\n");
  print("Cache-Control: no-cache\n");
  # send the redirect headers
  print("Status: 302 Logged Out\n");
  # with the start url
  print("Location: $start\n\n");

  return 0;
} # sub redirect_cookie

# the error page 
sub bad_page {
  my($query,$error) = (shift(),shift());
  my($htmlfile) = new IO::File('<' . ERRORPAGE);
  print $query->header()
    if defined($query) && ref($query) =~ /^CGI/;
  while ( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#ERROR\#\#/$error/g;
    print $line;
  }
  $htmlfile->close();

  return 0;
} # sub bad_page

# construct the url to redirect back to after login, based on
# information posted by the web server that redirected here
sub get_start {
  my($query) = shift();
  # get the server name and uri (path)
  my($starturl);
  if ( defined($query->param('url')) ) {
    $starturl = $query->param('url');
  } elsif(defined $query->param('server') and defined $query->param('uri')) {
    # if the port is not 80
    if(defined $query->param('port') and $query->param('port') != 80) {
      # add it to the server name
      $query->param('server', $query->param('server') . ':' . 
		    $query->param('port'));
    }
    # create the url from the server name and uri
    $starturl = 'http://' . $query->param('server') . 
			 $query->param('uri');
  } else {
    # if there's no info, use the default url
    $starturl = $cfg{'url'}{LOGGED_OUT};
  }
  return $starturl;
} # sub get_start
