#!/usr/bin/perl -w
# $Id: LPA.pl,v 1.4 2002/04/16 16:23:17 santana Exp $
use strict;

# 2.00.mario
#
# Lost Password Authority for macs
#
# TODO: find your password based on email as well as login
#       insert a verification page

use CGI;
use IO::Socket;
use IO::File;
use URI::Escape;
use DBI;
use Net::SMTP;

# the html pages
use constant LPAPAGE => 'LPA.html';
use constant RESULTPAGE => 'LPA_result.html';
use constant LPAEMAIL => 'LPA_email.txt';
use constant ERRORPAGE => 'LPA_error.html';

#
# Get configured
use Config::IniFiles;
my($cfg,%cfg);
die("Couldn't configure!")
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => "../macs.cfg",
		      -default => "default");
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

# the dbh will be global
my($dbh);

# find out which page to display based on extra path info
MAIN: {
  my($query) = new CGI();
  my($page) = $query->path_info();
  if($page =~ m/lostpw/) {
    # the passwd change form
    &lostpw_page($query);
  } elsif($page =~ m/result/) {
    # the password changer
    &result_page($query);
  } else {
    # go to the passwd change form by default
    &lostpw_page($query);
  }
  exit(0);
} # MAIN

#######################################################################
# generate the passwd change page
sub lostpw_page {
  my($query,$error) = (shift(),shift());
  $error='' unless defined($error);
  return error_page($query,'<hr><h1>No CGI object!</h1><hr>')
    unless ref($query) =~ '^CGI';

  # create the url to post the form to
  my($actionurl) = $query->url() . '/result';
  # get the start url to redirect back to
  my($starturl) = get_start($query);
  # are we told to display the URA?
  my($ura) = get_ura($query, $starturl);
  # open the html file
  my($htmlfile) = new IO::File('<' . LPAPAGE);
  return bad_page($query,'<hr><h1>Error opening '.LPAPAGE.'</h1><hr>')
    unless $htmlfile;

  # print the CGI headers
  print $query->header();
  # print the html file substituting the keywords
  while( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#ACTION\#\#/$actionurl/g;
    $line =~ s/\#\#START\#\#/$starturl/g;
    $line =~ s/\#\#ERROR\#\#/$error/g;
    $line =~ s/\#\#URA\#\#/$ura/g;
    print $line;
  }
  # close the html file
  $htmlfile->close();
  return 1;
} # lostpw_page

#######################################################################
# this checks the user provided login data
sub result_page {
  my($query) = shift();
  my($username);

  # give an error if no username was provided
  return lostpw_page($query,"You must provide a login.\n")
    unless defined($query->param('username')) && $query->param('username');
  $username = uri_escape($query->param('username'));

  # come up with a temporary passwd
  # FIXME: we should NOT use the first two chars of the passwd as the
  #   salt!
  srand();
  my($letter,@passwd,$passwd,$cryptpw);
  foreach $letter (0..7) {
    $passwd[$letter] = int(rand(26)+65);
    $passwd[$letter] = sprintf "%c", $passwd[$letter];
  }
  $passwd = join("",@passwd);
  $cryptpw = crypt($passwd, join("",@passwd[0,1]));
  
  # open a connection to the database
  $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p})
      unless defined($dbh) && $dbh->ping;
  return lostpw_page($query,"Couldn't connect to database")
      unless defined($dbh) && $dbh->ping;
  
  # Grab the user's info
  my($sth) = $dbh->prepare($cfg{'sql LPA'}{get_user});
  $sth->bind_param(1,$query->param('username'));
  $sth->execute();
  my($users) = $sth->fetchall_arrayref();
  if( $#$users < 0 ) {
    error_page($query,"could not find login " . $query->param('username'));
    exit;
  }
  if ( $#$users > 0 ) {
    error_page($query,"got multiple rows for login " .
	       $query->param('username') .
	       " in database");
    exit;
  }

  my($user) = { name   => $users->[0][0],
		email  => $users->[0][1],
		id     => $users->[0][2],
		passwd => $cryptpw,
		login  => $query->param('username') };

  # make sure there's a valid email
  error_page($query,"The login " . $user->{login} . " does not " .
	     "have a valid email associated with it.")
      unless defined($user->{email}) && $user->{email} =~ /.+@.+\..+/;
  
  # prepare the handle to update
  $sth = $dbh->prepare($cfg{'sql LPA'}{update_passwd});
  $sth->bind_param(1,$user->{passwd});
  $sth->bind_param(2,$user->{id});
    
  my($error);
  foreach my $key (keys %$user) {
    $error .= "user{$key} = $user->{$key}<br>";
  }
  warn($error);
  
  # change the password
  eval {$sth->execute;};
  if ( $@ ) {
    $error .= $DBI::errstr;
    return lostpw_page($query,"Error updating passwd: $error<br>\n");
  }

  # Send the message to the correct email
  my($smtp) = Net::SMTP->new($cfg{LPA}{mailhost});
  $smtp->mail($cfg{LPA}{mailfrom});
  $smtp->to($user->{email});
  $smtp->data();
  my($htmlfile) = new IO::File('<' . LPAEMAIL);  
  while( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#TO\#\#/$user->{email}/g;
    $line =~ s/\#\#USERNAME\#\#/$user->{login}/g;
    $line =~ s/\#\#PASSWD\#\#/$passwd/g;
    $line =~ s/\#\#PCA\#\#/$cfg{cgi}{PCA}/g;
    $smtp->datasend($line);
  }
  $htmlfile->close();
  $smtp->quit;

  # get the start url to redirect back to
  my($starturl) = get_start($query);
  # display the html result
  $htmlfile = new IO::File('<' . RESULTPAGE);
  return bad_page($query,'<hr><h1>Error opening '.RESULTPAGE.'</h1><hr>')
    unless $htmlfile;
  print $query->header();
  while( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#EMAIL\#\#/$user->{email}/g;
    $line =~ s/\#\#START\#\#/$starturl/g;
    print $line;
  }
  $htmlfile->close();
  return 1;  
} # sub result_page

#######################################################################
# the error page 
sub error_page {
  my($query,$error) = (shift(),shift());
  my($htmlfile) = new IO::File('<' . ERRORPAGE);
  print $query->header()
    if defined($query) && ref($query) =~ /^CGI/;
  while ( my $line = $htmlfile->getline() ) {
    $line =~ s/\#\#ERROR\#\#/$error/g;
    print $line;
  }
  $htmlfile->close();
  return 0;
} # sub error_page

#######################################################################
# construct the url to redirect back to after login, based on
# information posted by the web server that redirected here
sub get_start {
  my($query) = shift();
  # get the server name and uri (path)
  my($starturl);
  if ( defined($query->param('url')) ) {
    $starturl = $query->param('url');
  } elsif(defined $query->param('server') and defined $query->param('uri')) {
    # if the port is not 80
    if(defined $query->param('port') and $query->param('port') != 80) {
      # add it to the server name
      $query->param('server', $query->param('server') . ':' . 
		    $query->param('port'));
    }
    # create the url from the server name and uri
    $starturl = 'http://' . $query->param('server') . 
			 $query->param('uri');
  } else {
    # if there's no info, use the default url
    $starturl = $cfg{cgi}{LIA};
  }
  return $starturl;
} # sub get_start

#######################################################################
# should we link to the ura? default is no
sub get_ura {
  my($query) = shift();
  my($starturl) = shift();
  my($ura);
  if ( defined($query->param('ura')) && $query->param('ura') ) {
    $ura = '<form method=POST name=URAForm action="https://logindev.myco.com/URA">
              Go to our <input type=submit value="Self-Registration Page">
              <input type=hidden name=url value="' . $starturl . '">
            </form>';
  } else {
    $ura = 'Please contact the administrator of your Application for an account.<br>';
  }
  return $ura;
} # sub get_ura

