# Users:     1=macs,   2=guest,  3=test
# Groups:    1=Admins, 2=Guests, 3=Users
#   macs is in Admins, guest is is in Guests, all are in Users
# Resources: 1=/macs,  2=/macs/secure
#   Users can read 1&2, Guests can't read 2, Admins own 1


drop table macs_user_profile;
drop table macs_aid;
drop sequence macs_aid_id_seq;
drop table macs_user_map;
drop table macs_login_method;
drop sequence macs_login_method_id_seq;
drop table macs_resource_flag;
drop table macs_resource;
drop sequence macs_resource_id_seq;
drop table macs_group_member;
drop table macs_group;
drop sequence macs_group_id_seq;
drop table macs_user;
drop sequence macs_user_id_seq;


create sequence macs_user_id_seq;
create table macs_user (
	id int4 default nextval('macs_user_id_seq'),
	login text unique,
	passwd text,
	name text,
	email text,
	active int4 default 0 not null 
	       constraint macs_user_ck_active check (active in (0,1)),
	constraint macs_user_pk	primary key (id)
);

insert into macs_user(login,passwd,name,email,active)
	values('macs','macspw','macs admin','macs@medleysoft.com',1);
insert into macs_user(login,passwd,name,email,active)
	values('guest','guest','Guest User','guest@medleysoft.com',1);
insert into macs_user(login,passwd,name,email,active)
	values('test','test','Test User','test@medleysoft.com',1);


create sequence macs_group_id_seq;
create table macs_group (
	id int4 default nextval('macs_group_id_seq'),
	name text not null,
	constraint macs_group_pk primary key (id)
);

insert into macs_group(name) values('Admins');
insert into macs_group(name) values('Guests');
insert into macs_group(name) values('Users');


create table macs_group_member (
	group_id int4
		  not null
		  constraint macs_group_member_fk_group_id
			     references macs_group(id)
		  on delete cascade,
	member_user_id int4 default null
		  constraint macs_group_member_fk_member_user_id
			     references macs_user(id) on delete cascade,
	member_group_id int4 default null
		  constraint macs_group_member_fk_member_group_id
			     references macs_group(id) on delete cascade,
	constraint macs_group_member_ck_null check
		   ((member_user_id is null and member_group_id is not null)
		   or
		   (member_group_id is null and member_user_id is not null)),
	constraint macs_group_member_ck_self check
		   (group_id <> member_group_id),
	constraint macs_group_member_uq_all
		   unique (group_id,member_user_id,member_group_id)
);

insert into macs_group_member(group_id,member_user_id) 
	select g.id,u.id from macs_user u, macs_group g 
	where u.name='macs admin' and g.name='Users';
insert into macs_group_member(group_id,member_user_id) 
	select g.id,u.id from macs_user u, macs_group g 
	where u.name='macs admin' and g.name='Admins';
insert into macs_group_member(group_id,member_user_id) 
	select g.id,u.id from macs_user u, macs_group g 
	where u.name='Guest User' and g.name='Users';
insert into macs_group_member(group_id,member_user_id) 
	select g.id,u.id from macs_user u, macs_group g 
	where u.name='Guest User' and g.name='Guests';
insert into macs_group_member(group_id,member_user_id) 
	select g.id,u.id from macs_user u, macs_group g 
	where u.name='Test User' and g.name='Users';


create sequence macs_resource_id_seq;
create table macs_resource (
	id int4 default nextval('macs_resource_id_seq'),
	name text not null,
	parent int4 not null
	       constraint macs_resource_fk_resource
			  references macs_resource(id),
	constraint macs_resource_uq_parent unique (name,parent),
	constraint macs_resource_pk primary key (id)
);

insert into macs_resource(name,parent) values('macsweb',1);
insert into macs_resource(name,parent) values('secure',1);


create table macs_resource_flag (
	resource_id int4 not null 
	       constraint macs_resource_flag_fk_resource 
		            references macs_resource(id)
	       on delete cascade,
	group_id int4 not null 
	       constraint macs_resource_flag_fk_group 
			    references macs_group(id) 
	       on delete cascade,
	is_disabled int4 default 0 not null 
	       constraint macs_resource_flag_ck_disabled 
			   check (is_disabled in (0,1,2)),
	is_owner int4 default 0 not null 
	      constraint macs_resource_flag_ck_owner 
			   check (is_owner in (0,1,2)),
	is_admin int4 default 0 not null 
	     constraint macs_resource_flag_ck_admin 
			   check (is_admin in (0,1,2)),
	is_read int4 default 0 not null 
	     constraint macs_resource_flag_ck_read 
			   check (is_read in (0,1,2)),
	is_write int4 default 0 not null 
	      constraint macs_resource_flag_ck_write 
			    check (is_write in (0,1,2)),
	constraint macs_resource_flag_pk primary key (resource_id,group_id)
);

insert into macs_resource_flag(resource_id,group_id,is_disabled,is_owner,is_admin,is_read,is_write)
	values(1,1,2,1,1,1,1);
insert into macs_resource_flag(resource_id,group_id,is_disabled,is_owner,is_admin,is_read,is_write)
	values(1,2,2,2,2,1,2);
insert into macs_resource_flag(resource_id,group_id,is_disabled,is_owner,is_admin,is_read,is_write)
	values(2,2,0,0,0,2,0);
insert into macs_resource_flag(resource_id,group_id,is_disabled,is_owner,is_admin,is_read,is_write)
	values(1,3,2,2,2,1,2);


create sequence macs_login_method_id_seq;
create table macs_login_method (
	id int4 default nextval('macs_login_method_id_seq'),
	name text not null 
	     constraint macs_login_method_uq_name unique,
	constraint macs_login_method_pk	primary key (id)
);

insert into macs_login_method(name) values('internal');
insert into macs_login_method(name) values('test_LMSd');


create table macs_user_map (
	method_id int4 not null 
		constraint macs_user_map_fk_method
			   references macs_login_method(id) 
	        on delete cascade,
	user_id int4 not null 
		constraint macs_user_map_fk_user 
			   references macs_user(id) 
		on delete cascade,
	login text not null,
	constraint macs_user_map_pk primary key (method_id,user_id),
	constraint macs_user_map_uq_login unique (login,method_id)
);

insert into macs_user_map (method_id,user_id,login)
	select m.id,u.id,u.login from macs_login_method m, macs_user u
	where u.login='macs' and m.name='internal';
insert into macs_user_map (method_id,user_id,login)
	select m.id,u.id,u.login from macs_login_method m, macs_user u
	where u.login='guest' and m.name='internal';
insert into macs_user_map (method_id,user_id,login)
	select m.id,u.id,u.login from macs_login_method m, macs_user u
	where u.login='test' and m.name='internal';


create sequence macs_aid_id_seq;
create table macs_aid (
	id int4 default nextval('macs_aid_id_seq'),
	name text not null constraint macs_aid_uq_name unique,
	constraint macs_aid_pk primary key (id)
);

insert into macs_aid (name) values ('Internal');


create table macs_user_profile (
	user_id int4
		constraint macs_user_profile_fk_user 
			   references macs_user(id) 
                on delete cascade,
	aid_id int4 
	       constraint macs_user_profile_fk_aid 
			  references macs_aid(id) 
	       on delete cascade,
	name text not null,
	value text,
	constraint macs_user_profile_pk primary key (user_id,aid_id,name)
);

