#!/usr/bin/perl -w
#
# $Id: RCA.pl,v 1.4 2002/04/16 04:59:43 santana Exp $
#
# 04/26/99 Blake Mitchell
# 02.00.mario
#
# RCA.pl (Registration Change Authority)
# Auto Profile Management CGI for macs
#

use CGI;
# send errors back to the browser and to the log
use CGI::Carp qw(fatalsToBrowser);
# love that database
use DBI;
# to read the html files
use IO::File;
# you're not paranoid, everyone really is out to get you
use strict;

# the profile editing form 
use constant EDITPROFILEPAGE => "RCA_EP.html";
# the password changing form
use constant CHANGEPASSWDPAGE => "RCA_CP.html";
# the error page
use constant ERRORPAGE => "RCA_ERROR.html";

#
# Get configured
use Config::IniFiles;
my($cfg,%cfg);
die("Couldn't configure!")
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => "../macs.cfg",
		      -default => "default");
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

# global db handle so we don't connect for each query
my($dbh);

# the main block
MAIN: {
  # get the CGI parameters
  my($query) = new CGI();

  # get the extra path information
  my($page) = $query->path_info();

  # check the extra path information to see which page to display
  if($page =~ m/profile/) {
    # go to the edit profile page
    &ep_page($query);
  } elsif($page =~ m/presult/) {
    # go to the change password results page
    &ep_result($query);
  } elsif($page =~ m/passwd/) {
    # go to the change password results page
    &cp_page($query);
  } elsif($page =~ m/cresult/) {
    # go to the change password results page
    &cp_result($query);
  } else {
    # go to the edit profile page by default
    &ep_page($query);
  }
  exit(0);
} # MAIN

######################################################################
# generate the profile editing page
sub ep_page {
  my($query,$error) = (shift(),shift());
  $error = "" unless defined($error);

  # create the url to post the form to
  my($actionurl) = $query->url() . "/presult";

  # construct the url to redirect back to after profile editing, based
  # on information posted by the web server that redirected here
  my($starturl) = get_starturl($query);

  # Grab the user's info and "update" it with info from the query if
  # it exists
  my($user)=get_userinfo($query,$ENV{'HTTP_'.uc($cfg{LMS}{http_uid_header})});
  my($key);
  foreach $key (keys %$user) {
    if ( defined($query->param($key)) && $query->param($key) ) {
      $user->{$key} = $query->param($key);
    }
  }
  
  # print the CGI headers
  print $query->header();

  # open the html file
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . EDITPROFILEPAGE);
  my($line);

  # print the html file substituting in the action url and start url
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#NAME\#\#/$user->{"name"}/e;
    $line =~ s/\#\#LOGIN\#\#/$user->{"login"}/e;
    $line =~ s/\#\#EMAIL\#\#/$user->{"email"}/e;
    $line =~ s/\#\#ACTION\#\#/$actionurl/g;
    $line =~ s/\#\#START\#\#/$starturl/g;
    $line =~ s/\#\#ERROR\#\#/$error/g;
    print $line;
  }
  # close the html file
  $htmlfile->close();
} # sub ep_page

######################################################################
# process the profile editing page
sub ep_result {
  my($query) = shift();

  # Do the passwd and passwdck fields match?
  return ep_page($query,"Your passwords did not match")
      if $query->param("passwd") ne $query->param("passwdck");

  # Grab the user's info
  my($user)=get_userinfo($query,$ENV{'HTTP_'.uc($cfg{LMS}{http_uid_header})});
  
  # Find things to update, create sql to execute
  my(@updating);
  if ( defined($query->param("name")) && $query->param("name")
       && $query->param("name") ne $user->{"name"} ) {
    push(@updating,"Name");
    $user->{name} = $query->param("name");
  }

  # FIXME: we should confirm a change of email by changing and
  # sending a new passwd to the new email
  if ( defined($query->param("email")) && $query->param("email")
       && $query->param("email") ne $user->{"email"} ) {
    push(@updating,"Email");
    $user->{email} = $query->param("email");
  }

  # FIXME: we should NOT use the first two chars of the passwd as the
  #   salt!
  # note that we checked for passwd == paswdck above
  if ( defined($query->param("passwd")) && $query->param("passwd") ) {
    push(@updating,"Password");
    $user->{passwd} = crypt($query->param("passwd"),
			    substr($query->param("passwd"),0,2));
  }
  
  if ( @updating ) {
    $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p})
	unless defined($dbh) && $dbh->ping;
    return ep_page($query,"Couldn't connect to database")
	unless defined($dbh) && $dbh->ping;

    # prepare the handle to update
    my($sth) = $dbh->prepare($cfg{'sql RCA'}{update_user});
    $sth->bind_param(1,$user->{name});
    $sth->bind_param(2,$user->{email});
    $sth->bind_param(3,$user->{login});
    $sth->bind_param(4,$user->{password});
    $sth->bind_param(5,$user->{id});
    
    # Do the dew
    my($rv) = $sth->execute;
    unless ( $rv ) {      
      return ep_page($query,"Error updating ".join(", ",@updating));
    }
  }
  
  return redirect_back($query);
} # sub ep_result

######################################################################
# generate the change passwd page
sub cp_page {
  my($query,$error) = (shift(),shift());
  $error = "" unless defined($error);

  # create the url to post the form to
  my($actionurl) = $query->url() . "/cresult";

  # construct the url to redirect back to after profile editing, based
  # on information posted by the web server that redirected here
  my($starturl) = get_starturl($query);

  # print the CGI headers
  print $query->header();

  # open the html file
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . CHANGEPASSWDPAGE);
  my($line);

  # print the html file substituting in the action url and start url
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#ACTION\#\#/$actionurl/g;
    $line =~ s/\#\#START\#\#/$starturl/g;
    $line =~ s/\#\#ERROR\#\#/$error/g;
    print $line;
  }
  # close the html file
  $htmlfile->close();
} # sub cp_page

######################################################################
# process the profile editing page
sub cp_result {
  my($query) = shift();

  return cp_page($query,"You must supply a new password.")
      unless defined($query->param("passwd")) && $query->param("passwd");

  return cp_page($query,"Your passwords did not match")
      if $query->param("passwd") ne $query->param("passwdck");

  # Grab the user's info
  my($user)=get_userinfo($query,$ENV{'HTTP_'.uc($cfg{LMS}{http_uid_header})});

  # open a connection to the database
  $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p})
      unless defined($dbh) && $dbh->ping;
  return cp_page($query,"Couldn't connect to database")
      unless defined($dbh) && $dbh->ping;
  
  # FIXME: we should NOT use the first two chars of the passwd as the
  #   salt!
  $user->{passwd} = crypt($query->param("passwd"),
			  substr($query->param("passwd"),0,2));

  # prepare the handle to update
  my($sth) = $dbh->prepare($cfg{'sql RCA'}{update_user});
  $sth->bind_param(1,$user->{name});
  $sth->bind_param(2,$user->{email});
  $sth->bind_param(3,$user->{login});
  $sth->bind_param(4,$user->{passwd});
  $sth->bind_param(5,$user->{id});
    
  # Do the dew
  my($rv) = $sth->execute;
  return ep_page($query,"Error updating password")
      unless $rv;
  
  return redirect_back($query);
} # sub cp_result

######################################################################
# the error page 
sub error_page {
  my($query,$error) = (shift(),shift());
  $error = "" unless defined($error);

  # get the server name and uri (path)
  # embellish the info here instead of the html file in case we don't
  # find it (we can tell we didn't find it because it's the default)
  my($starturl) = get_starturl($query);
  if ( $starturl eq $cfg{url}{LOGGED_IN} ) {
    $starturl = "";
  } else {
    $starturl = 'It looks like you were ' .
  	'<a href="' . $starturl . '">here</a> ' .
  	'before coming here to change your profile.<p>';
  }

  # print the CGI headers
  print $query->header();

  # do the dew
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . ERRORPAGE);
  my($line);
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#STARTURL\#\#/$starturl/;
    $line =~ s/\#\#ERROR\#\#/$error/;
    print $line;
  }
  $htmlfile->close();
} # sub error_page

######################################################################
# construct the url to redirect back to after profile editing, based
# on information posted by the web server that redirected here
sub get_starturl {
  my($query) = shift();
  # get the server name and uri (path)
  if(defined $query->param("server") and defined $query->param("uri")) {
    # if the port is not 80
    if(defined $query->param("port") and $query->param("port") != 80) {
      # add it to the server name
      $query->param("server", $query->param("server") . ':' . 
		    $query->param("port"));
    }
    # create the url from the server name and uri
    return "http://" . $query->param("server") . $query->param("uri");
  } elsif ( defined $query->param("url") ) {
    return $query->param("url");
  } else {
    # if there's no info, use the default url
    return $cfg{url}{LOGGED_IN};
  }
} # sub get_starturl

######################################################################
# returns a user's macs profile as a hashref
sub get_userinfo {
  my($query,$uid) = (shift(),shift());
  return { 
    name => "",
    login => "",
    email => "",
    passwd => "",
    id => ""
  } unless defined($uid);
  $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p})
      unless defined($dbh) && $dbh->ping;
  return error_page($query,"database connection error")
      unless defined($dbh) && $dbh->ping;
  my($sth) = $dbh->prepare($cfg{'sql RCA'}{get_info_complete});
  $sth->bind_param(1,$uid);
  $sth->execute();
  my($users) = $sth->fetchall_arrayref();
  if( $#$users < 0 ) {
    error_page($query,"could not find uid ".$uid." in database");
    exit;
  }
  if ( $#$users > 0 ) {
    error_page($query,"got multiple rows for uid ".$uid." in database");
    exit;
  }
  return { name   => $users->[0][0],
	   login  => $users->[0][1],
	   email  => $users->[0][2],
	   id     => $users->[0][3],
	   passwd => $users->[0][4] };
} # sub get_userinfo

######################################################################
# spits the headers to redirect after changing profile
#   if we're just redirecting to $cfg{url}{LOGGED_IN}, insert some message
#   stating how the update was successful
sub redirect_back {
  my($query) = shift();
  my($url) = get_starturl($query);
  return ep_page($query,"Profile update successful.")
      if $url eq $cfg{url}{LOGGED_IN};
  print("Status: 302 Successful Profile Change\n");
  print("Location: $url\n\n");
  return 0;
} # sub redirect_back
