#!/usr/bin/perl -w
# $Id: SFSd.pl,v 1.1 2002/03/25 09:19:54 santana Exp $
#
# SFSd - SOAP Frontend Server
#
# A SOAP frontend interface service for macs
#

package SFSd;
use strict;
use IO::Socket;
use SOAP::Transport::HTTP;

#####################################
## initialize process

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || "../macs.cfg");
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => "default" );
die("Couldn't configure: $!\n")
    unless $cfg->Sections(); 
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

$SIG{__WARN__} = sub { ;; };
$SIG{TERM} = sub { warn("Got TERM\n"); unlink $cfg{SFS}{pidf}; exit(0); };
$SIG{HUP} = sub { warn("Got HUP\n"); unlink $cfg{SFS}{pidf}; exit(0); };

FORK: {
  # the process id variable
  my($pid);
  # spawn a child
  if($pid = fork()) {
    # parent process here, bye bye
    exit(0);
  } elsif(defined $pid) {
    # child process here, we continue
    last FORK;
  } elsif($! =~ m/[Nn]o more process/) {
    # can't fork, sleep and try again
    sleep(5);
    redo FORK;
  } else {
    # unrecoverable fork error
    die("Couldn't fork: $!\n");
  }
}


#####################################
## handlers

sub aus {
    shift;
    warn("Entering aus.\n");
    my($aus);
    $aus = new IO::Socket( Domain => AF_INET,
			   PeerAddr => $cfg{AUS}{s} );
    return "ERR could not connect to an AUS\n"
	unless $aus && $aus->peerport();
    my $cmd = join(" ", @_); chomp $cmd;
    $aus->{sock}->write($cmd."\n",length($cmd)+1);
    return $aus->{sock}->getline();
} # sub aus

sub acs {
    shift;
    warn("Entering acs.\n");
    my($acs);
      $acs = new IO::Socket( Domain => AF_INET,
			     PeerAddr => $cfg{ACS}{s} );
    return "ERR could not connect to an ACS\n"
	unless $acs && $acs->peerport();
    my $op = $_[0];
    my $cmd = join(" ", @_); chomp $cmd;
    $acs->write($cmd."\n",length($cmd)+1);
    my $rv = $acs->getline();
    if ( $op eq "GET" && $rv =~ /^\s*YES\s+([0-9]+)\s*/o ) {
	$acs->read($rv,$1,length($rv));
    }
    return $rv;
} # sub acs

sub ats {
    warn("Entering ats.\n");
    my($ats);
    $ats = new IO::Socket( Domain => AF_INET,
			   PeerAddr => $cfg{SFS}{s} );
    return "ERR could not connect to an ATS\n"
	unless $ats && $ats->peerport();
    my $cmd = join(" ", @_); chomp $cmd;
    $ats->{sock}->write($cmd."\n",length($cmd)+1);
    return $ats->{sock}->getline();
} # sub ats

sub ups {
    warn("Entering ups.\n");
    my($ups,$rv);
    $ups = new IO::Socket( Domain => AF_INET,
			   PeerAddr => $cfg{UPS}{s} );
    return "ERR could not connect to an UPS\n"
	unless $ups && $ups->peerport();
    my $cmd = join(" ", @_); chomp $cmd;
    $ups->write($cmd."\n",length($cmd)+1);
    return $ups->getline();
} # sub ups

sub hw() {
    return "Hello, world.\n";
} # sub hw

sub quit() {
    warn("Got QUIT.\n");
    unlink $cfg{SFS}{pidf};
    exit(0);
} # sub quit

sub echo() {
    shift;
    return join(',',@_)."\n"
} # sub echo

#####################################
## go!

die("$cfg{SFS}{pidf} exists!\n")
    if -x $cfg{SFS}{pidf};
die("Error opening $cfg{SFS}{pidf} for overwrite\n")
    unless open(PIDF,">".$cfg{SFS}{pidf});
print PIDF $$;
close PIDF;

while (1) {
    print "Going...";
    my($daemon) = SOAP::Transport::HTTP::Daemon
	->new(LocalAddr => $cfg{SFS}{s}, Reuse => 1)
	->dispatch_to('SFSd');
    print "Contact to SOAP server at ", $daemon->url, "\n";
    $daemon->handle();
  warn("SOAP::Transport::HTTP::Daemon returned. Restarting.\n");
  last;
}

