#!/usr/bin/perl -w
# $Id: UPSd.pl,v 1.5 2002/03/25 04:36:29 santana Exp $
#
# TODO: add SET and DEL commands

package UPSd;
use strict;
use lib "../lib";
use Tie::DistHash;
use Net::muxDaemon;
# retrieve all the info from the SQL DB
use DBI qw(:sql_types);

#
# Get configured
use Config::IniFiles;
my($cfg,%cfg);
die('Couldn\'t configure')
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => '../macs.cfg',
		      -default => 'default');
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

# make sure we're running in the background
FORK: {
  # the process id variable
  my($pid);
  # spawn a child
  if($pid = fork()) {
    # parent process here, bye bye
    exit(0);
  } elsif(defined $pid) {
    # child process here, we continue
    last FORK;
  } elsif($! =~ m/[Nn]o more process/) {
    # can't fork, sleep and try again
    sleep(2);
    redo FORK;
  } else {
    # unrecoverable fork error
    die('Couldn\'t fork');
  }
}

# has the net::demon opened syslog yet?
my($syslog_open) = 0;

# database specific variables
my($dbh, $info_sth, $core_sth, $set_sth);
# connect to the database
&db_setup();

#
# write a config file for the auc
open(CCF, '>../upc.cfg');
print CCF 'HTTP_UID_HEADER ',$cfg{AUS}{http_uid_header},"\n";
print CCF 'TIMEOUT ',$cfg{UPS}{timeout},"\n";
#
# The hosts in our distributed setup
my($hosts) = [];
for ( $cfg->val('UPS','p') ) {
    print CCF 'UPS ',$_,"\n";
    push(@$hosts, {addr => $_});
}
close CCF;

#
# the DistHash database
my($db) = {};
my($ch) = tie( %$db, 'Tie::DistHash',
	       ckptfile => $cfg{UPS}{ckptf},
	       sync => $hosts );

# starting net::demon, syslog will be open
$syslog_open = 1;
# vroom...
new Net::muxDaemon( POOL => $hosts,
		    database => $db,
		    cache => $ch,
		    LOGTAG => $cfg{UPS}{logtag},
		    PIDFILE => $cfg{UPS}{pidf}
		    );
die("Net::muxDaemons don't return!\n\t$!\n\t$@\n");

# set up the database connection
sub db_setup {
  # make sure the database isn't already connected
  eval { $dbh->disconnect(); };
  # create the database handle
  $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p});
  # make sure the database is connected
  unless($dbh) {
    if($syslog_open) {
      syslog('err', "Couldn't connect: $DBI::errstr");
      $KILLED = 1;
    } else {
      die("Couldn't connect: $DBI::errstr");
    }
  }
  # the database should wine if there are any errors
  $dbh->{RaiseError} = 1;
  # prepare the query to retrieve group information
  $@ = "";
  # trap errors
  eval {
    $info_sth   = $dbh->prepare($cfg{'sql UPS'}{get_user_info});
    $core_sth   = $dbh->prepare($cfg{'sql UPS'}{get_user_core});
    $set_sth    = $dbh->prepare($cfg{'sql UPS'}{set_user_info});
  };
  # exit if there are errors
  if($@){
    if($syslog_open) {
      syslog('err', "$@");
      $KILLED = 1;
    } else {
      die("$@");
    }
  }
}

# retreive all profile items for all applications for this user
# returns a hashref where ->{aid}{item_key} = value
sub db_get_info {
    my($uid) = shift();
    
    # the results of the database query
    my($info) = {};
    # if a database operation fails, reconnect and try again
    my($second_try) = 0;
  DB: {
      $@ = "";
      # trap errors
      eval {
	  $info_sth->bind_param(1, $uid, { TYPE => SQL_INTEGER });
	  $info_sth->execute();
	  my($aid,$pkey,$value);
	  while ( ($aid,$pkey,$value) = $info_sth->fetchrow_array() ) {
	      $info->{$aid}{$pkey} = $value;
	  }
	  $info_sth->finish();
	  $core_sth->bind_param(1, $uid, { TYPE => SQL_INTEGER });
	  $core_sth->execute();
	  my($login,$name,$email) = $core_sth->fetchrow_array();
	  $core_sth->finish();
	  if ( defined($login) ) {
	      $info->{1}{LOGIN} = $login;
	      $info->{1}{EMAIL} = defined($email) ? $email : '';
	      $info->{1}{NAME}  = defined($name)  ? $name  : '';
	  }
      };
      # if there are errors
      if($@){
	  # log the error
	  syslog('err', "$@");
	  # try again
	  unless($second_try) {
	      $second_try = 1;
	      &db_setup();
	      redo DB;
	  }
	  # bye bye
	  $KILLED = 1;
	  return {};
      }
  }
    return $info;
}

#
# Sets a profile item in the database
sub db_set_info {
    my($this,$uid,$aid,$pkey,$val) =
	(shift(),shift(),shift(),shift(),shift());
    # if a database operation fails, reconnect and try again
    my($second_try) = 0;
  DB: {
      $@ = "";
      # trap errors
      eval {
	  # bind the parameters to the query
	  $set_sth->bind_param(1, $uid,  { TYPE => SQL_INTEGER });
	  $set_sth->bind_param(2, $aid,  { TYPE => SQL_INTEGER });
	  $set_sth->bind_param(3, $pkey, { TYPE => SQL_VARCHAR });
	  $set_sth->bind_param(4, $val,  { TYPE => SQL_VARCHAR });
	  # execute the query
	  $set_sth->execute();
	  $set_sth->finish();
      };
      # if there are errors
      if ($@) {
	  # try again
	  unless ($second_try) {
	      syslog('err',"$@");
	      $second_try = 1;
	      &db_setup();
	      redo DB;
	  }
	  # bye bye
	  syslog('err',"Second try: $@");
	  $KILLED = 1;
	  return 0;
      }
      $this->{database}{$uid}{$aid}{$pkey} = $val;
      return 1;
  }
}    

  
#
#
#
sub process_command {
    my($this,$buf) = (shift(),shift());
    syslog('debug',"Entering process_command: $buf");
    my(@command);
    my($reply) = '';
    my($aid,$uid,$pkey,$operation,$len);
    if ( $buf =~ m/^\s*$/ ) {
	syslog('debug','Exiting process_command: OK NOOP');
	return "OK NOOP\n";
    }  
    
    # trim the white space off the command string
    $buf =~ s/^\s*|\s*$//;

    # split up the command string on spaces
    @command = split(' ', $buf);
    # the operation to perform
    $operation = uc(shift(@command));
    
  OP: {
      
      # LIST aid uid
      if ( $operation eq 'LIST' ) {
	  ($aid,$uid) = (shift(@command),shift(@command));
	  unless ( $uid ) {
	      $reply = "ERR LIST: not enough arguments\n";
	      last OP;
	  }
	  if (!exists($this->{database}{$uid})) {	
	      $this->{database}{$uid} = &db_get_info($uid);
	  }
	  if (!exists($this->{database}{$uid}{$aid})) {
	      $reply = "NO profile for uid $uid aid $aid\n";
	  } else {
	      $reply = 'YES ' .
		  join(' ', keys(%{$this->{database}{$uid}{$aid}})) . "\n";
	  }
      }

      # GET aid uid key
      elsif ($operation eq 'GET') {
	  ($aid,$uid,$pkey) =
	      (shift(@command),shift(@command),shift(@command));
	  unless ( $uid ) {
	      $reply = "ERR GET: not enough arguments\n";
	      last OP;
	  }
	  if ( !exists($this->{database}{$uid}{$aid}{$pkey}) ) {
	      my($user_info) = &db_get_info($uid);
	      $this->{database}{$uid} = $user_info
		  if $user_info;
	      if ( !exists($this->{database}{$uid}{$aid}{$pkey}) ) {
		  $reply = "NO key:$pkey for uid:$uid in app:$aid\n";
		  last OP;
	      }
	  }
	  $reply = 'YES ' .
	      length($this->{database}{$uid}{$aid}{$pkey}) .
	      "\n" . $this->{database}{$uid}{$aid}{$pkey};
      }

      # PUT aid uid key len\nvalue
      elsif ( $operation eq 'PUT' ) {
	  ($aid,$uid,$pkey,$len) = (shift(@command),shift(@command),
				    shift(@command),shift(@command));
	  unless ( defined($aid) && defined($uid) &&
		   defined($pkey) && $len ) {
	      $reply = "ERR PUT: not enough arguments\n";
	      last OP;
	  }
	  # FIXME: deal w/ missing aid
	  my($val);
	  my($client) = shift();
	  unless ($this->getmore($client,$len,\$val)) {
	      $reply = "ERR no value read\n";
	      last OP;
	  }
	  # deal w/ dupes
	  if (exists($this->{database}{$uid}{$aid}{$pkey})) {
	      $reply = "NO duplicate entries. Use SET.\n";
	      last OP;
	  } else {
	      my($user_info) = &db_get_info($uid);
	      $this->{database}{$uid} = $user_info
		  if $user_info;
	      if (exists($this->{database}{$uid}{$aid}{$pkey})) {
		  $reply = "NO duplicate entries. Use SET.\n";
		  last OP;
	      }
	  }
	  unless (db_set_info($this,$uid,$aid,$pkey,$val)) {
	      $reply = "ERR setting value $@\n";
	      last OP;
	  }
	  $reply = "OK\n";
      }
      
      # PING
      elsif ( $operation eq 'PING' ) {
	  $reply = "PONG\n";
      }

      # QUIT
      elsif ( $operation eq 'QUIT' ) {
	  $reply = "OK\n";
	  $KILLED = 1;
      }

      # menospeakie
      else {
	  $reply = "ERR unknown command: $operation $buf\n";
      }

  } # OP
    
    syslog('debug',"Exiting process_command: $reply");
    return $reply;
} # sub process_command

