#!/usr/bin/perl -w
# $Id: URA.pl,v 1.6 2002/04/16 05:02:17 santana Exp $
#
# Blake Mitchell
# .mario
#
# URA.pl (User Registration Authority)
# Auto Registration CGI for macs
#

use CGI;
# send errors back to the browser and to the log
use CGI::Carp qw(fatalsToBrowser);
# love that database
use DBI qw(:sql_types);
# your window to the outside
use IO::Socket;
# to read the html files
use IO::File;
# prevent mangling of cgi parameters
use URI::Escape;
# you're not paranoid, everyone really is out to get you
use strict;

#
# Get configured
use Config::IniFiles;
my($cfg,%cfg);
die("Couldn't configure!")
    unless $cfg = tie(%cfg, 'Config::IniFiles',
		      -file => "../macs.cfg",
		      -default => "default");
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

# the selfreg html page
use constant AUTOREGPAGE => "URA.html";
# the bad passwd error html page
use constant URAERRORPAGE => "URA_error.html";
# the duped email html page
use constant DUPEMAILPAGE => "URA_dupemail.html";

# the main block
MAIN: {
  # get the CGI parameters
  my($query) = new CGI();
  # get the extra path information
  my($page) = $query->path_info();
  # check the extra path information to see which page to display
  if($page =~ m/register/) {
    # go to the autoregistration page
    ar_page($query);
  } elsif($page =~ m/result/) {
    # go to the autoregistration results page
    result_page($query);
  } elsif($page =~ m/withdupes/) {
    # go straight to adding the user -- no dupes checking
    result_page($query);
  } else {
    # go to the autoregistration page by default
    ar_page($query);
  }
  exit(0);
} # MAIN


# generate the autoregistration page
sub ar_page {
  my($query,$error) = (shift(),shift());
  $error = ''
      if !defined($error);
  # open the html file
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . AUTOREGPAGE);
  my($line);
  # create the url to post the form to
  my($actionurl) = $query->url() . "/result";

  my($starturl) = get_start($query);

  # print the CGI headers
  print $query->header();
  # print the html file substituting the appropriate keywords
  # FIXME: catalog/document these
  my $name = ($query->param("name") || '');
  my $email = ($query->param("email") || '');
  my $login = ($query->param("login") || '');
  my $passwd = ($query->param("passwd") || '');
  my $passwdck = ($query->param("passwdck") || '');
  
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#NAME\#\#/$name/e;
    $line =~ s/\#\#EMAIL\#\#/$email/e;
    $line =~ s/\#\#LOGIN\#\#/$login/e;
    $line =~ s/\#\#PASSWD\#\#/$passwd/e;
    $line =~ s/\#\#PASSWDCK\#\#/$passwdck/e;
    $line =~ s/\#\#ACTION\#\#/$actionurl/g;
    $line =~ s/\#\#START\#\#/$starturl/g;
    $line =~ s/\#\#ERROR\#\#/$error/g;
    print $line;
  }
  # close the html file
  $htmlfile->close();
} # sub ar_page


# this inserts the user-provided registration info
sub result_page {
  my($query) = shift();
  my($error) = "";
  my($login);
  
  # give an error if no login
  unless(defined $query->param("login") and
	 $login = $query->param("login")) {
    $error .= "No login was provided.<br>\n";
  }
  # whine if there's an illegal character in it
  if ( $login =~ s/([^A-Za-z0-9._-]+)/<font color="#000000">$1<\/font>/g ) {
    $query->param(-name=>'login',-value=>'');
    $error .= "Logins may only contain alphanumerics, " .
	"periods, underscores, and dashes: $login<br>\n";
  }

  # give an error if no name
  unless (defined $query->param("name") and
	 $query->param("name")) {
    $error .= "No name was provided.<br>\n";
  }
  # give an error if no email
  unless (defined $query->param("email") and
	 $query->param("email")) {
    $error .= "No email was provided.<br>\n";
  }
  # give an error if no passwd
  unless (defined($query->param("passwd")) and
	 $query->param("passwd") ) {
    $error .= "Your passwords do not match.<br>\n";
  }
  # error out if passwords don't match
  unless ($query->param("passwd") eq $query->param("passwdck")) {
    $error .= "Your passwords do not match.<br>\n";
  }
  if ( $error ) {
    ar_page($query,
	    "<font size=3>Please correct the following errors:</font><p>" .
	    $error . "<hr>");
    return;
  }

  my($dbh) = undef;
  # check the uniqueness of the email unless they're have the
  #   "withdupes" path-info
  if ($query->path_info() !~ m/withdupes/) {
    eval {
      # open a connection to the database
      $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p});
      croak("Couldn't connect: $DBI::errstr") unless $dbh;
      # the database should whine if there are any errors
      $dbh->{RaiseError} = 1;
      # but don't print them out!
      $dbh->{PrintError} = 0;

      my($sth) = $dbh->prepare($cfg{'sql URA'}{get_username_from_email});
      $sth->bind_param(1,$query->param("email"));
      $sth->execute;
      my($rv) = $sth->fetchall_arrayref();
      if ( $#$rv > -1 ) {
        # we found some dupes!
        dup_email($query,$rv);
        exit;
      }
    } 
  }

  # did we crash?
  if ( $@ ) {
    bad_page($query,"Looking for email dupes() " . $@);
    exit 0;
  }
  
  return adduser_page($query,$dbh);
} # sub result_page


# Log the new user in, then feed the browser a cookie and
# redirect it to the start url
sub redirect_cookie {
  my($query,$uid) = (shift(),shift());
  my($redir) = get_start($query);

  my($lms,$skey);
  # open a connection to the LMS
  return bad_page($query,"<hr><h1>Couldn't connect to LMS: $!</h1><hr>")
      unless $lms = new IO::Socket::INET( PeerAddr => $cfg{LMS}{s} );

  # dew the doo
  $lms->print("AUTH internal ",
	      $query->param("login"), ' ',
	      $query->param("passwd"), ' ',
	      $query->remote_addr(), ' ',
	      $lms->sockport(), "\n");
  my($rv) = $lms->getline();
  warn($rv);
  # (BAM) fixed broken ifs
  if ($rv =~ m/^YES\s+(\S+)\s*(.*)$/i) {
    $skey = $1;
  } elsif ($rv =~ m/^NO\s?(.*)$/i) {
    return bad_page($query,'<hr><h1>Authentication denied!</h1><hr>');
  } else {
    return bad_page($query,"<hr><h1>Login Method Server failed</h1>$rv<hr>");
  }

  # format the cookie
  my($authcookie) = $query->cookie(-name   => $cfg{LMS}{cookie_name}, 
				   -value  => $skey, 
				   -path   => '/', 
				   -domain => $cfg{LMS}{cookie_domain});
  # set the cookie
  warn("Set-cookie: " . 
	$cfg{LMS}{cookie_name} . "=" . $skey . "; " .
	"domain=" . $cfg{LMS}{cookie_domain} . "; " .
	"path=/\n");
  warn("Set-cookie: $authcookie\n");
  print("Set-cookie: $authcookie\n");
  # send the redirect headers
  print("Status: 302 Moved\n");
  # with the start url
  print("Location: $redir\n\n");

  return 0;
} # sub redirect_cookie


# the error page 
sub bad_page {
  my($query,$error) = (shift(),shift());
  
  my($starturl) = get_start($query);

  # print the CGI headers
  print $query->header();
  
  # spit the file
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . URAERRORPAGE);
  my($line);
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#STARTURL\#\#/$starturl/;
    $line =~ s/\#\#ERROR\#\#/$error/;
    print $line;
  }
  $htmlfile->close();
} # bad_page

# construct the url to redirect back to after login, based on
# information posted by the web server that redirected here
sub get_start {
  my($query) = shift();
  # get the server name and uri (path)
  my($starturl);
  if ( defined($query->param('url')) ) {
    $starturl = $query->param('url');
  } elsif(defined $query->param('server') and defined $query->param('uri')) {
    # if the port is not 80
    if(defined $query->param('port') and $query->param('port') != 80) {
      # add it to the server name
      $query->param('server', $query->param('server') . ':' . 
		    $query->param('port'));
    }
    # create the url from the server name and uri
    $starturl = 'http://' . $query->param('server') . 
 			 $query->param('uri');
    warn("query.server: $starturl");
  } else {
    # if there's no info, use the default url
    $starturl = $cfg{url}{LOGGED_IN};
  }
  return $starturl;
} # sub get_start

# prints the duped email confirmation page
sub dup_email {
  my($query,$rv) = (shift(),shift());

  my($start) = get_start($query);
  my($dupes) = join("\n", map {
                            "<li><a href=\"/LPA/result?username=" .
				$_->[0] . "\">" . $_->[0] . "</a>"
			  } @$rv);
  my $name = ($query->param("name") || '');
  my $email = ($query->param("email") || '');
  my $login = ($query->param("login") || '');
  my $passwd = ($query->param("passwd") || '');
  
  my($confirm) = $query->url."/withdupes";

  # print the CGI headers
  print $query->header();
  # spit the file
  my($htmlfile) = new IO::File;
  $htmlfile->open("<" . DUPEMAILPAGE);
  my($line);
  while($line = $htmlfile->getline()) {
    $line =~ s/\#\#NAME\#\#/$name/e;
    $line =~ s/\#\#EMAIL\#\#/$email/e;
    $line =~ s/\#\#LOGIN\#\#/$login/e;
    $line =~ s/\#\#PASSWD\#\#/$passwd/e;
    $line =~ s/\#\#PASSWDCK\#\#/$passwd/e;
    $line =~ s/\#\#DUPELIST\#\#/$dupes/g;
    $line =~ s/\#\#CONFIRM\#\#/$confirm/g;
    $line =~ s/\#\#STARTURL\#\#/$start/g;
    $line =~ s/\#\#LIA\#\#/$cfg{cgi}{LIA}/g;
    $line =~ s/\#\#ADMIN\#\#/$cfg{LMS}{admin}/g;
    print $line;
  }
  $htmlfile->close();
  return 1;
} # sub dup_email

# add the user without checking for duped emails
sub adduser_page {
  my($query,$dbh) = (shift(),shift());

  # were we passed a $dbh?
  eval {
    # open a connection to the database
    $dbh = DBI->connect($cfg{db}{d}, $cfg{db}{l}, $cfg{db}{p});
    croak("Couldn't connect: $DBI::errstr") unless $dbh;
    # the database should wine if there are any errors
    $dbh->{RaiseError} = 1;
    # but don't print them out!
    $dbh->{PrintError} = 0;
  } if !$dbh;
  # did we crash?
  if ( $@ ) {
    bad_page($query,"Add user: " . $@);
  }

  my($sth);
  eval {
    my($cryptpw) = crypt($query->param("passwd"), 
			 substr($query->param("passwd"),0,2) );
    # prepare the sp call
    $sth = $dbh->prepare($cfg{'sql URA'}{add_user});
    $sth->bind_param(1,$query->param("name"));
    $sth->bind_param(2,$query->param("email"));
    $sth->bind_param(3,$query->param("login"));
    $sth->bind_param(4,$cryptpw);
    $sth->execute;
    $sth = $dbh->prepare($cfg{'sql URA'}{get_userid_from_username});
    $sth->bind_param(1,$query->param("login"));
    $sth->execute;
    my($uids) = $sth->fetchall_arrayref();
    die("Duplicate logins in database!\n")
	if $#$uids != 0;
    my($uid) = $uids->[0][0];
    $sth = $dbh->prepare($cfg{'sql URA'}{map_internal_user});
    $sth->bind_param(1,$uid);
    $sth->bind_param(2,$query->param("login"));
    $sth->execute();
    $sth = $dbh->prepare($cfg{'sql URA'}{add_user_to_group});
    $sth->bind_param(1,$uid);
    $sth->bind_param(2,'Guests');
    $sth->execute();
    $sth->finish();
    # redirect to the start page
    redirect_cookie($query,$uid);
    # disconnect the database
    $dbh->disconnect();
  };
  # did we crash?
  if ( $@ ) {
    if ( $sth->errstr =~ /duplicate.*login/i ) {
      ar_page($query,
	      "The login you chose is taken. " .
	      "Please choose another.<br>\n");
    } else {
      bad_page($query,"adduser_page: " . $@);
    }
  }
} # sub adduser_page  

