#!/usr/bin/perl -w
# $Id: XRSd.pl,v 1.2 2001/08/07 01:17:14 santana Exp $
#
# XRSd - XML-RPC Server
#
# An XML-RPC frontend interface service for macs
#
# 10.99.mario
#

package XRSd;
use strict;
use lib "../lib";
use IO::Socket;
use Frontier::Daemon;

#####################################
## initialize process

#
# Get configured
use Config::IniFiles;
my($INI) = (shift() || "../macs.cfg");
my($cfg,%cfg);
die("Config file $INI unreadable.\n")
    unless -r $INI;
$cfg = tie( %cfg, 'Config::IniFiles',
	    -file => $INI,
	    -default => "default" );
die("Couldn't configure: $!\n")
    unless $cfg->Sections(); 
for ( keys %{$cfg{env}} ) {
  $ENV{$_} = $cfg{env}{$_};
}

$SIG{__WARN__} = sub { ;; };
$SIG{TERM} = sub { warn("Got TERM\n"); unlink $cfg{XRS}{pidf}; exit(0); };
$SIG{HUP} = sub { warn("Got HUP\n"); unlink $cfg{XRS}{pidf}; exit(0); };

FORK: {
  # the process id variable
  my($pid);
  # spawn a child
  if($pid = fork()) {
    # parent process here, bye bye
    exit(0);
  } elsif(defined $pid) {
    # child process here, we continue
    last FORK;
  } elsif($! =~ m/[Nn]o more process/) {
    # can't fork, sleep and try again
    sleep(5);
    redo FORK;
  } else {
    # unrecoverable fork error
    die("Couldn't fork: $!\n");
  }
}

#####################################
## initialize sockets

#
# The hosts in our distributed setup
my($ACSs) = []; for ($cfg->val("ACS","p")) {push(@$hosts, {addr => $_});}
my($AUSs) = []; for ($cfg->val("AUS","p")) {push(@$hosts, {addr => $_});}
my($ATSs) = []; for ($cfg->val("ATS","p")) {push(@$hosts, {addr => $_});}
my($UPSs) = []; for ($cfg->val("UPS","p")) {push(@$hosts, {addr => $_});}

sub init_AUSs {
  for ( @$AUSs ) {
    $_->{sock} = new IO::Socket( Domain => AF_INET,
				 PeerAddr => $_->{addr} );
  }
} # sub init_AUSs

sub init_ACSs {
  for ( @$ACSs ) {
    $_->{sock} = new IO::Socket( Domain => AF_INET,
				 PeerAddr => $_->{addr} );
  }
} # sub init_ACSs

sub init_ATSs {
  for ( @$ATSs ) {
    $_->{sock} = new IO::Socket( Domain => AF_INET,
				 PeerAddr => $_->{addr} );
  }
} # sub init_ATSs

sub init_UPSs {
  for ( @$UPSs ) {
    $_->{sock} = new IO::Socket( Domain => AF_INET,
				 PeerAddr => $_->{addr} );
  }
} # sub init_UPSs


#####################################
## handlers

sub runAUS {
  warn("Entering runAUS.\n");
  my($aus,$rv);
  for (my($count)=0; $count <= $#$AUSs; $count++) {
    $aus = $AUSs->[$count];
    unless ( $aus->{sock} && $aus->{sock}->peerport() == $aus->{port} ) {
      $aus->{sock} = new IO::Socket( Domain => AF_INET,
				     PeerAddr => $aus->{host},
				     PeerPort => $aus->{port}
				     );
    }
    last
      if $aus->{sock} && $aus->{sock}->peerport() == $aus->{port};
  }
  return "ERR could not connect to an AUS\n"
    unless $aus->{sock} && $aus->{sock}->peerport() == $aus->{port};
  my $cmd = join(" ", @_); chomp $cmd;
  $aus->{sock}->write($cmd."\n",length($cmd)+1);
  return $aus->{sock}->getline();
} # sub runAUS

sub runACS {
  warn("Entering runACS. - ",$#$ACSs,"\n");
  my($acs);
  for (my($count)=0; $count <= $#$ACSs; $count++) {
    $acs = $ACSs->[$count];
    unless ( $acs->{sock} && $acs->{sock}->opened ) {
      $acs->{sock} = new IO::Socket( Domain => AF_INET,
				     PeerAddr => $acs->{host},
				     PeerPort => $acs->{port}
				   );
    }
    $acs->{sock} && $acs->{sock}->opened && last;
  }
  return "ERR could not connect to an ACS\n"
    unless $acs->{sock} && $acs->{sock}->opened;
  my $op = $_[0];
  my $cmd = join(" ", @_); chomp $cmd;
  $acs->{sock}->write($cmd."\n",length($cmd)+1);
  my $rv = $acs->{sock}->getline();
  if ( $op eq "GET" && $rv =~ /^\s*YES\s+([0-9]+)\s*/o ) {
    $acs->{sock}->read($rv,$1,length($rv));
  }
  return $rv;
} # sub runACS

sub runATS {
  warn("Entering runATS.\n");
  my($ats);
  for (my($count)=0; $count <= $#$ATSs; $count++) {
    $ats = $ATSs->[$count];
    unless ( $ats->{sock} && $ats->{sock}->peerport() == $ats->{port} ) {
      $ats->{sock} = new IO::Socket( Domain => AF_INET,
				     PeerAddr => $ats->{host},
				     PeerPort => $ats->{port}
				   );
    }
    last
      if $ats->{sock}->opened;
  }
  return "ERR could not connect to an ATS\n"
    unless $ats->{sock}->opened;
  my $cmd = join(" ", @_); chomp $cmd;
  $ats->{sock}->write($cmd."\n",length($cmd)+1);
  return $ats->{sock}->getline();
} # sub runATS

sub runUPS {
  warn("Entering runUPS.\n");
  my($ups,$rv);
  for (my($count)=0; $count <= $#$UPSs; $count++) {
    $ups = $UPSs->[$count];
    unless ( $ups->{sock} && $ups->{sock}->peerport() == $ups->{port} ) {
      $ups->{sock} = new IO::Socket( Domain => AF_INET,
				     PeerAddr => $ups->{host},
				     PeerPort => $ups->{port}
				   );
    }
    last
      if $ups->{sock} && $ups->{sock}->peerport() == $ups->{port};
  }
  return "ERR could not connect to an UPS\n"
    unless $ups->{sock} && $ups->{sock}->peerport() == $ups->{port};
  my $cmd = join(" ", @_); chomp $cmd;
  $ups->{sock}->write($cmd."\n",length($cmd)+1);
  return $ups->{sock}->getline();
} # sub runUPS


#####################################
## go!

die("$cfg{XRS}{pidf} exists!\n")
    if -x $cfg{XRS}{pidf};
die("Error opening $cfg{XRS}{pidf} for overwrite\n")
    unless open(PIDF,">".$cfg{XRS}{pidf});
print PIDF $$;
close PIDF;

&init_AUSs();
&init_ACSs();
&init_ATSs();
&init_UPSs();

while (1) {
  new Frontier::Daemon
      ( LocalAddr => $LocalAddr,
	Reuse => 1,
	methods => { AUS => \&runAUS,
		     ACS => \&runACS,
		     ATS => \&runATS,
		     UPS => \&runUPS,
		     ECHO => sub { return join(",",@_),"\n"; },
		     QUIT => sub { warn("Got QUIT.\n");
				   unlink $cfg{XRS}{pidf};
				   exit(0); } }
	);
  warn("Frontier::Daemon returned. Restarting.\n");
  last;
}

